@@ -1,4 +1,4 @@
-/* $Id: cgps.c 4074 2006-12-04 21:55:45Z jfrancis $ */
+/* $Id: cgps.c 4547 2007-12-13 01:38:22Z esr $ */
 /*
  * Copyright (c) 2005 Jeff Francis <jeff@gritch.org>
  *
@@ -64,9 +64,6 @@
    "standard" 80x24 screen. */
 #define SATELLITES_WIDTH 35
 
-/* This is the title to put at the top of the screen. */
-#define TITLE "GPSD Test Client"
-
 /* ================================================================
    You shouldn't have to modify any #define values below this line.
    ================================================================ */
@@ -96,11 +93,15 @@
 #include <math.h>
 #include <errno.h>
 #include <assert.h>
-
-#include <ncurses.h>
 #include <signal.h>
 
 #include "gpsd_config.h"
+#ifdef HAVE_NCURSES_H
+#include <ncurses.h>
+#else
+#include <curses.h>
+#endif /* HAVE_NCURSES_H */
+
 #include "gps.h"
 
 static struct gps_data_t *gpsdata;
@@ -114,7 +115,6 @@ static char *speedunits = "mph";
 
 static WINDOW *datawin, *satellites, *messages;
 
-static int title_flag=0;
 static int raw_flag=0;
 static int silent_flag=0;
 static int fixclear_flag=0;
@@ -125,7 +125,7 @@ static int window_length;
 static int display_sats;
 
 /* Function to call when we're all done.  Does a bit of clean-up. */
-static void die(int sig UNUSED) 
+static void die(int sig UNUSED)
 {
   /* Ignore signals. */
   (void)signal(SIGINT,SIG_IGN);
@@ -153,9 +153,9 @@ static enum deg_str_type deg_type = deg_dd;
 /* This gets called once for each new sentence until we figure out
    what's going on and switch to either update_gps_panel() or
    update_compass_panel(). */
-static void update_probe(struct gps_data_t *gpsdata, 
+static void update_probe(struct gps_data_t *gpsdata,
 		       char *message,
-		       size_t len UNUSED , 
+		       size_t len UNUSED ,
 		       int level UNUSED)
 {
   /* Send an 'i' once per second until we figure out what the GPS
@@ -180,57 +180,205 @@ static void update_probe(struct gps_data_t *gpsdata,
 }
 
 
-/* This gets called once for each new compass sentence. */
-static void update_compass_panel(struct gps_data_t *gpsdata, 
-                         char *message,
-                         size_t len UNUSED , 
-                         int level UNUSED)
-{
-  char *s;
+/*@ -globstate @*/
+static void windowsetup(void){
+  /* Set the window sizes per the following criteria:
 
+     1.  Set the window size to display the maximum number of
+     satellites possible, but not more than the size required to
+     display the maximum number of satellites gpsd is capable of
+     tracking (MAXCHANNELS - 2).
+
+     2.  If the screen size will not allow for the full complement of
+     satellites to be displayed, set the windows sizes smaller, but
+     not smaller than the number of lines necessary to display all of
+     the fields in the 'datawin'.  The list of displayed satellites
+     will be truncated to fit the available window size.  (TODO: If
+     the satellite list is truncated, omit the satellites not used to
+     obtain the current fix.)
+
+     3.  If the screen is large enough to display all possible
+     satellites (MAXCHANNELS - 2) with space still left at the bottom,
+     add a window at the bottom in which to scroll raw gpsd data.
+  */
+  int xsize, ysize;
+
+  getmaxyx(stdscr,ysize,xsize);
+
+  if(compass_flag==1) {
+    if(ysize == MIN_COMPASS_DATAWIN_SIZE) {
+      raw_flag = 0;
+      window_length = MIN_COMPASS_DATAWIN_SIZE;
+    } else if(ysize > MIN_COMPASS_DATAWIN_SIZE) {
+      raw_flag = 1;
+      window_length = MIN_COMPASS_DATAWIN_SIZE;
+    } else {
+      (void)mvprintw(0, 0, "Your screen must be at least 80x%d to run cgps.",MIN_COMPASS_DATAWIN_SIZE);
+      /*@ -nullpass @*/
+      (void)refresh();
+      /*@ +nullpass @*/
+      (void)sleep(5);
+      die(0);
+    }
+  } else {
+    if(ysize == MAX_SATWIN_SIZE) {
+      raw_flag = 0;
+      window_length = MAX_SATWIN_SIZE;
+      display_sats = MAX_POSSIBLE_SATS;
+    } else if(ysize == MAX_SATWIN_SIZE + 1) {
+      raw_flag = 1;
+      window_length = MAX_SATWIN_SIZE;
+      display_sats = MAX_POSSIBLE_SATS;
+    } else if(ysize > MAX_SATWIN_SIZE + 2) {
+      raw_flag = 1;
+      window_length = MAX_SATWIN_SIZE;
+      display_sats = MAX_POSSIBLE_SATS;
+    } else if(ysize > MIN_GPS_DATAWIN_SIZE) {
+      raw_flag = 0;
+      window_length = ysize - raw_flag;
+      display_sats = window_length - SATWIN_OVERHEAD - raw_flag;
+    } else if(ysize == MIN_GPS_DATAWIN_SIZE) {
+      raw_flag = 0;
+      window_length = MIN_GPS_DATAWIN_SIZE;
+      display_sats = window_length - SATWIN_OVERHEAD - 1;
+    } else {
+      (void)mvprintw(0, 0, "Your screen must be at least 80x%d to run cgps.",MIN_GPS_DATAWIN_SIZE);
+      /*@ -nullpass @*/
+      (void)refresh();
+      /*@ +nullpass @*/
+      (void)sleep(5);
+      die(0);
+    }
+  }
+
+  /* Set up the screen for either a compass or a gps receiver. */
+  if(compass_flag==1) {
+    /* We're a compass, set up accordingly. */
+
+    /*@ -onlytrans @*/
+    datawin    = newwin(window_length, DATAWIN_WIDTH, 0, 0);
+    (void)nodelay(datawin,(bool)TRUE);
+    if(raw_flag==1) {
+      messages   = newwin(0, 0, window_length, 0);
+
+      /*@ +onlytrans @*/
+      (void)scrollok(messages, true);
+      (void)wsetscrreg(messages, 0, ysize - (window_length));
+    }
+
+    /*@ -nullpass @*/
+    (void)refresh();
+    /*@ +nullpass @*/
+
+    /* Do the initial field label setup. */
+    (void)mvwprintw(datawin, 1, DATAWIN_DESC_OFFSET, "Time:");
+    (void)mvwprintw(datawin, 2, DATAWIN_DESC_OFFSET, "Heading:");
+    (void)mvwprintw(datawin, 3, DATAWIN_DESC_OFFSET, "Pitch:");
+    (void)mvwprintw(datawin, 4, DATAWIN_DESC_OFFSET, "Roll:");
+    (void)mvwprintw(datawin, 5, DATAWIN_DESC_OFFSET, "Dip:");
+    (void)mvwprintw(datawin, 6, DATAWIN_DESC_OFFSET, "Rcvr Type:");
+    (void)wborder(datawin, 0, 0, 0, 0, 0, 0, 0, 0);
+
+  } else {
+    /* We're a GPS, set up accordingly. */
+
+    /*@ -onlytrans @*/
+    datawin    = newwin(window_length, DATAWIN_WIDTH, 0, 0);
+    satellites = newwin(window_length, SATELLITES_WIDTH, 0, DATAWIN_WIDTH);
+    (void)nodelay(datawin,(bool)TRUE);
+    if(raw_flag==1) {
+      messages   = newwin(ysize - (window_length), xsize, window_length, 0);
+
+      /*@ +onlytrans @*/
+      (void)scrollok(messages, true);
+      (void)wsetscrreg(messages, 0, ysize - (window_length));
+    }
+
+    /*@ -nullpass @*/
+    (void)refresh();
+    /*@ +nullpass @*/
+
+    /* Do the initial field label setup. */
+    (void)mvwprintw(datawin, 1, DATAWIN_DESC_OFFSET, "Time:");
+    (void)mvwprintw(datawin, 2, DATAWIN_DESC_OFFSET, "Latitude:");
+    (void)mvwprintw(datawin, 3, DATAWIN_DESC_OFFSET, "Longitude:");
+    (void)mvwprintw(datawin, 4, DATAWIN_DESC_OFFSET, "Altitude:");
+    (void)mvwprintw(datawin, 5, DATAWIN_DESC_OFFSET, "Speed:");
+    (void)mvwprintw(datawin, 6, DATAWIN_DESC_OFFSET, "Heading:");
+    (void)mvwprintw(datawin, 7, DATAWIN_DESC_OFFSET, "Climb:");
+    (void)mvwprintw(datawin, 8, DATAWIN_DESC_OFFSET, "Status:");
+    (void)mvwprintw(datawin, 9, DATAWIN_DESC_OFFSET, "GPS Type:");
+
+    /* Note that the following four fields are exceptions to the
+       sizing rule.  The minimum window size does not include these
+       fields, if the window is too small, they get excluded.  This
+       may or may not change if/when the output for these fields is
+       fixed and/or people request their permanance.  They're only
+       there in the first place because I arbitrarily thought they
+       sounded interesting. ;^) */
+
+    if(window_length >= (MIN_GPS_DATAWIN_SIZE + 4)) {
+      (void)mvwprintw(datawin, 10, DATAWIN_DESC_OFFSET, "Horizontal Err:");
+      (void)mvwprintw(datawin, 11, DATAWIN_DESC_OFFSET, "Vertical Err:");
+      (void)mvwprintw(datawin, 12, DATAWIN_DESC_OFFSET, "Course Err:");
+      (void)mvwprintw(datawin, 13, DATAWIN_DESC_OFFSET, "Speed Err:");
+    }
+
+    (void)wborder(datawin, 0, 0, 0, 0, 0, 0, 0, 0);
+    (void)mvwprintw(satellites, 1,1, "PRN:   Elev:  Azim:  SNR:  Used:");
+    (void)wborder(satellites, 0, 0, 0, 0, 0, 0, 0, 0);
+  }
+}
+/*@ +globstate @*/
+
+/* This gets called once for each new compass sentence. */
+static void update_compass_panel(struct gps_data_t *gpsdata,
+			char *message,
+			size_t len UNUSED ,
+			int level UNUSED)
+{
+  char scr[128];
   /* Print time/date. */
-  (void)wmove(datawin, 1, DATAWIN_VALUE_OFFSET);
   if (isnan(gpsdata->fix.time)==0) {
-    char scr[128];
-    (void)wprintw(datawin,"%s",unix_to_iso8601(gpsdata->fix.time, scr, (int)sizeof(s)));
+    (void)unix_to_iso8601(gpsdata->fix.time, scr, sizeof(scr));
   } else
-    (void)wprintw(datawin,"n/a                    ");
+    (void)snprintf(scr, sizeof(scr), "n/a");
+  (void)mvwprintw(datawin, 1, DATAWIN_VALUE_OFFSET, "%-*s", 27, scr);
 
   /* Fill in the heading. */
-  (void)wmove(datawin, 2, DATAWIN_VALUE_OFFSET);
   if (isnan(gpsdata->fix.track)==0) {
-    (void)wprintw(datawin,"%.1f     ", gpsdata->fix.track);
+    (void)snprintf(scr, sizeof(scr), "%.1f", gpsdata->fix.track);
   } else
-    (void)wprintw(datawin,"n/a         ");
+    (void)snprintf(scr, sizeof(scr), "n/a");
+  (void)mvwprintw(datawin, 2, DATAWIN_VALUE_OFFSET, "%-*s", 27, scr);
 
   /* Fill in the pitch. */
-  (void)wmove(datawin, 3, DATAWIN_VALUE_OFFSET);
   if (isnan(gpsdata->fix.climb)==0) {
-    (void)wprintw(datawin,"%.1f     ", gpsdata->fix.climb);
+    (void)snprintf(scr, sizeof(scr), "%.1f", gpsdata->fix.climb);
   } else
-    (void)wprintw(datawin,"n/a         ");
+    (void)snprintf(scr, sizeof(scr), "n/a");
+  (void)mvwprintw(datawin, 3, DATAWIN_VALUE_OFFSET, "%-*s", 27, scr);
 
   /* Fill in the roll. */
-  (void)wmove(datawin, 4, DATAWIN_VALUE_OFFSET);
   if (isnan(gpsdata->fix.speed)==0)
-    (void)wprintw(datawin,"%.1f     ",gpsdata->fix.speed);
+    (void)snprintf(scr, sizeof(scr), "%.1f",gpsdata->fix.speed);
   else
-    (void)wprintw(datawin,"n/a         ");
+    (void)snprintf(scr, sizeof(scr), "n/a");
+  (void)mvwprintw(datawin, 4, DATAWIN_VALUE_OFFSET, "%-*s", 27, scr);
 
   /* Fill in the speed. */
-  (void)wmove(datawin, 5, DATAWIN_VALUE_OFFSET);
   if (isnan(gpsdata->fix.altitude)==0)
-    (void)wprintw(datawin,"%.1f     ", gpsdata->fix.altitude);
+    (void)snprintf(scr, sizeof(scr), "%.1f", gpsdata->fix.altitude);
   else
-    (void)wprintw(datawin,"n/a         ");
+    (void)snprintf(scr, sizeof(scr), "n/a");
+  (void)mvwprintw(datawin, 5, DATAWIN_VALUE_OFFSET, "%-*s", 27, scr);
 
   /* Fill in receiver type. */
-  (void)wmove(datawin, 6, DATAWIN_VALUE_OFFSET);
-  if(got_gps_type==1) {
-    (void)wprintw(datawin,"%s",gps_type);
-  } else {
-    (void)wprintw(datawin,"unknown     ");
-  }
+  if(got_gps_type==1)
+    (void)snprintf(scr, sizeof(scr), "%s",gps_type);
+  else
+    (void)snprintf(scr, sizeof(scr), "unknown");
+  (void)mvwprintw(datawin, 6, DATAWIN_VALUE_OFFSET, "%-*s", 27, scr);
 
   /* Be quiet if the user requests silence. */
   if(silent_flag==0 && raw_flag==1) {
@@ -245,14 +393,14 @@ static void update_compass_panel(struct gps_data_t *gpsdata,
 
 
 /* This gets called once for each new GPS sentence. */
-static void update_gps_panel(struct gps_data_t *gpsdata, 
-                         char *message,
-                         size_t len UNUSED , 
-                         int level UNUSED)
+static void update_gps_panel(struct gps_data_t *gpsdata,
+			char *message,
+			size_t len UNUSED ,
+			int level UNUSED)
 {
-  int i,n,c;
+  int i,n;
   int newstate;
-  char *s;
+  char scr[128];
 
   /* This is for the satellite status display.  Originally lifted from
      xgps.c.  Note that the satellite list may be truncated based on
@@ -261,126 +409,125 @@ static void update_gps_panel(struct gps_data_t *gpsdata,
   if (gpsdata->satellites!=0) {
     if (display_sats >= MAX_POSSIBLE_SATS) {
       for (i = 0; i < MAX_POSSIBLE_SATS; i++) {
-	(void)wmove(satellites, i+2, 1);
 	if (i < gpsdata->satellites) {
-	  (void)wprintw(satellites," %3d    %02d    %03d    %02d      %c  ",
+	    (void)snprintf(scr, sizeof(scr), 
+			 " %3d    %02d    %03d    %02d      %c",
 			gpsdata->PRN[i],
 			gpsdata->elevation[i], gpsdata->azimuth[i],
 			gpsdata->ss[i], gpsdata->used[i] ? 'Y' : 'N');
 	} else {
-	  for(c = 0; c < SATELLITES_WIDTH - 3; c++) {
-	    (void)wprintw(satellites," ");
-	  }
+          (void)strlcpy(scr, "", sizeof(scr));
 	}
+	(void)mvwprintw(satellites, i+2, 1, "%-*s", SATELLITES_WIDTH - 3, scr);
       }
     } else {
       n=0;
       for (i = 0; i < MAX_POSSIBLE_SATS; i++) {
 	if (n < display_sats) {
-	  (void)wmove(satellites, n+2, 1);
 	  if ((i < gpsdata->satellites) && ((gpsdata->used[i]!=0) || (gpsdata->satellites <= display_sats))) {
-	    n++;
-	    (void)wprintw(satellites," %3d    %02d    %03d    %02d      %c  ",
+	      (void)snprintf(scr, sizeof(scr),
+			  " %3d    %02d    %03d    %02d      %c",
 			  gpsdata->PRN[i],
 			  gpsdata->elevation[i], gpsdata->azimuth[i],
 			  gpsdata->ss[i], gpsdata->used[i] ? 'Y' : 'N');
+	    (void)mvwprintw(satellites, n+2, 1, "%-*s", SATELLITES_WIDTH - 3, scr);
+	    n++;
 	  }
 	}
       }
-      
+
       if(n < display_sats) {
-	for(i = n; i <= display_sats; i++) {
-	  (void)wmove(satellites, i+2, 1);
-	  for(c = 0; c < SATELLITES_WIDTH - 3; c++) {
-	    (void)wprintw(satellites," ");
-	  }
+	for (i = n; i <= display_sats; i++) {
+	  (void)mvwprintw(satellites, i+2, 1, "%-*s", SATELLITES_WIDTH - 3, "");
 	}
       }
-      
+
     }
   }
-  
+
   /* Print time/date. */
-  (void)wmove(datawin, 1, DATAWIN_VALUE_OFFSET);
   if (isnan(gpsdata->fix.time)==0) {
-    char scr[128];
-    (void)wprintw(datawin,"%s",unix_to_iso8601(gpsdata->fix.time, scr, (int)sizeof(s)));
+    (void)unix_to_iso8601(gpsdata->fix.time, scr, sizeof(scr));
   } else
-    (void)wprintw(datawin,"n/a                    ");
+    (void)snprintf(scr, sizeof(scr), "n/a");
+  (void)mvwprintw(datawin, 1, DATAWIN_VALUE_OFFSET, "%-*s", 27, scr);
+
 
   /* Fill in the latitude. */
-  (void)wmove(datawin, 2, DATAWIN_VALUE_OFFSET);
   if (gpsdata->fix.mode >= MODE_2D && isnan(gpsdata->fix.latitude)==0) {
-    s = deg_to_str(deg_type,  fabs(gpsdata->fix.latitude));
-    (void)wprintw(datawin,"%s %c     ", s, (gpsdata->fix.latitude < 0) ? 'S' : 'N');
+    (void)snprintf(scr, sizeof(scr), "%s %c", 
+         deg_to_str(deg_type,  fabs(gpsdata->fix.latitude)), 
+         (gpsdata->fix.latitude < 0) ? 'S' : 'N');
   } else
-    (void)wprintw(datawin,"n/a         ");
+    (void)snprintf(scr, sizeof(scr), "n/a");
+  (void)mvwprintw(datawin, 2, DATAWIN_VALUE_OFFSET, "%-*s", 27, scr);
 
   /* Fill in the longitude. */
-  (void)wmove(datawin, 3, DATAWIN_VALUE_OFFSET);
   if (gpsdata->fix.mode >= MODE_2D && isnan(gpsdata->fix.longitude)==0) {
-    s = deg_to_str(deg_type,  fabs(gpsdata->fix.longitude));
-    (void)wprintw(datawin,"%s %c     ", s, (gpsdata->fix.longitude < 0) ? 'W' : 'E');
+    (void)snprintf(scr, sizeof(scr), "%s %c", 
+         deg_to_str(deg_type,  fabs(gpsdata->fix.longitude)), 
+         (gpsdata->fix.longitude < 0) ? 'W' : 'E');
   } else
-    (void)wprintw(datawin,"n/a         ");
+    (void)snprintf(scr, sizeof(scr), "n/a");
+  (void)mvwprintw(datawin, 3, DATAWIN_VALUE_OFFSET, "%-*s", 27, scr);
 
   /* Fill in the altitude. */
-  (void)wmove(datawin, 4, DATAWIN_VALUE_OFFSET);
   if (gpsdata->fix.mode == MODE_3D && isnan(gpsdata->fix.altitude)==0)
-    (void)wprintw(datawin,"%.1f %s     ",gpsdata->fix.altitude*altfactor, altunits);
+    (void)snprintf(scr, sizeof(scr), "%.1f %s",gpsdata->fix.altitude*altfactor, altunits);
   else
-    (void)wprintw(datawin,"n/a         ");
+    (void)snprintf(scr, sizeof(scr), "n/a");
+  (void)mvwprintw(datawin, 4, DATAWIN_VALUE_OFFSET, "%-*s", 27, scr);
 
   /* Fill in the speed. */
-  (void)wmove(datawin, 5, DATAWIN_VALUE_OFFSET);
   if (gpsdata->fix.mode >= MODE_2D && isnan(gpsdata->fix.track)==0)
-    (void)wprintw(datawin,"%.1f %s     ", gpsdata->fix.speed*speedfactor, speedunits);
+    (void)snprintf(scr, sizeof(scr), "%.1f %s", gpsdata->fix.speed*speedfactor, speedunits);
   else
-    (void)wprintw(datawin,"n/a         ");
+    (void)snprintf(scr, sizeof(scr), "n/a");
+  (void)mvwprintw(datawin, 5, DATAWIN_VALUE_OFFSET, "%-*s", 27, scr);
 
   /* Fill in the heading. */
-  (void)wmove(datawin, 6, DATAWIN_VALUE_OFFSET);
   if (gpsdata->fix.mode >= MODE_2D && isnan(gpsdata->fix.track)==0)
-    (void)wprintw(datawin,"%.1f degrees     ", gpsdata->fix.track);
+    (void)snprintf(scr, sizeof(scr), "%.1f degrees", gpsdata->fix.track);
   else
-    (void)wprintw(datawin,"n/a          ");
+    (void)snprintf(scr, sizeof(scr), "n/a");
+  (void)mvwprintw(datawin, 6, DATAWIN_VALUE_OFFSET, "%-*s", 27, scr);
 
   /* Fill in the rate of climb. */
-  (void)wmove(datawin, 7, DATAWIN_VALUE_OFFSET);
   if (gpsdata->fix.mode == MODE_3D && isnan(gpsdata->fix.climb)==0)
-    (void)wprintw(datawin,"%.1f %s/min     "
-                  , gpsdata->fix.climb * altfactor * 60, altunits);
+    (void)snprintf(scr, sizeof(scr), "%.1f %s/min",
+	gpsdata->fix.climb * altfactor * 60, altunits);
   else
-    (void)wprintw(datawin,"n/a         ");
-  
+    (void)snprintf(scr, sizeof(scr), "n/a");
+  (void)mvwprintw(datawin, 7, DATAWIN_VALUE_OFFSET, "%-*s", 27, scr);
+
   /* Fill in the GPS status and the time since the last state
      change. */
-  (void)wmove(datawin, 8, DATAWIN_VALUE_OFFSET);
   if (gpsdata->online == 0) {
     newstate = 0;
-    (void)wprintw(datawin,"OFFLINE          ");
+    (void)snprintf(scr, sizeof(scr), "OFFLINE");
   } else {
     newstate = gpsdata->fix.mode;
     switch (gpsdata->fix.mode) {
     case MODE_2D:
-      (void)wprintw(datawin,"2D %sFIX (%d secs)   ",(gpsdata->status==STATUS_DGPS_FIX)?"DIFF ":"", (int) (time(NULL) - status_timer));
+      (void)snprintf(scr, sizeof(scr), "2D %sFIX (%d secs)",(gpsdata->status==STATUS_DGPS_FIX)?"DIFF ":"", (int) (time(NULL) - status_timer));
       break;
     case MODE_3D:
-      (void)wprintw(datawin,"3D %sFIX (%d secs)   ",(gpsdata->status==STATUS_DGPS_FIX)?"DIFF ":"", (int) (time(NULL) - status_timer));
+      (void)snprintf(scr, sizeof(scr), "3D %sFIX (%d secs)",(gpsdata->status==STATUS_DGPS_FIX)?"DIFF ":"", (int) (time(NULL) - status_timer));
       break;
     default:
-      (void)wprintw(datawin,"NO FIX (%d secs)     ", (int) (time(NULL) - status_timer));
+      (void)snprintf(scr, sizeof(scr), "NO FIX (%d secs)", (int) (time(NULL) - status_timer));
       break;
     }
   }
+  (void)mvwprintw(datawin, 8, DATAWIN_VALUE_OFFSET, "%-*s", 27, scr);
 
   /* Fill in the receiver type. */
-  (void)wmove(datawin, 9, DATAWIN_VALUE_OFFSET);
   if(got_gps_type==1) {
-    (void)wprintw(datawin,"%s",gps_type);
+    (void)snprintf(scr, sizeof(scr), "%s",gps_type);
   } else {
-    (void)wprintw(datawin,"unknown     ");
+    (void)snprintf(scr, sizeof(scr), "unknown");
   }
+  (void)mvwprintw(datawin, 9, DATAWIN_VALUE_OFFSET, "%-*s", 27, scr);
 
     /* Note that the following four fields are exceptions to the
        sizing rule.  The minimum window size does not include these
@@ -393,32 +540,32 @@ static void update_gps_panel(struct gps_data_t *gpsdata,
     if(window_length >= (MIN_GPS_DATAWIN_SIZE + 4)) {
 
       /* Fill in the estimated horizontal position error. */
-      (void)wmove(datawin, 10, DATAWIN_VALUE_OFFSET + 5);
       if (isnan(gpsdata->fix.eph)==0)
-	(void)wprintw(datawin,"+/- %d %s     ", (int) (gpsdata->fix.eph * altfactor), altunits);
+	(void)snprintf(scr, sizeof(scr), "+/- %d %s", (int) (gpsdata->fix.eph * altfactor), altunits);
       else
-	(void)wprintw(datawin,"n/a         ");
-      
+        (void)snprintf(scr, sizeof(scr), "n/a");
+      (void)mvwprintw(datawin, 10, DATAWIN_VALUE_OFFSET + 5, "%-*s", 22, scr);
+
       /* Fill in the estimated vertical position error. */
-      (void)wmove(datawin, 11, DATAWIN_VALUE_OFFSET + 5);
       if (isnan(gpsdata->fix.epv)==0)
-	(void)wprintw(datawin,"+/- %d %s     ", (int)(gpsdata->fix.epv * altfactor), altunits);
+	(void)snprintf(scr, sizeof(scr), "+/- %d %s", (int)(gpsdata->fix.epv * altfactor), altunits);
       else
-	(void)wprintw(datawin,"n/a         ");
-      
+        (void)snprintf(scr, sizeof(scr), "n/a");
+      (void)mvwprintw(datawin, 11, DATAWIN_VALUE_OFFSET + 5, "%-*s", 22, scr);
+
       /* Fill in the estimated track error. */
-      (void)wmove(datawin, 12, DATAWIN_VALUE_OFFSET + 5);
       if (isnan(gpsdata->fix.epd)==0)
-	(void)wprintw(datawin,"+/- %.1f deg     ", (gpsdata->fix.epd));
+	(void)snprintf(scr, sizeof(scr), "+/- %.1f deg", (gpsdata->fix.epd));
       else
-	(void)wprintw(datawin,"n/a          ");
-      
+        (void)snprintf(scr, sizeof(scr), "n/a");
+      (void)mvwprintw(datawin, 12, DATAWIN_VALUE_OFFSET + 5, "%-*s", 22, scr);
+
       /* Fill in the estimated speed error. */
-      (void)wmove(datawin, 13, DATAWIN_VALUE_OFFSET + 5);
       if (isnan(gpsdata->fix.eps)==0)
-	(void)wprintw(datawin,"+/- %d %s     ", (int)(gpsdata->fix.eps * speedfactor), speedunits);
+	(void)snprintf(scr, sizeof(scr), "+/- %d %s", (int)(gpsdata->fix.eps * speedfactor), speedunits);
       else
-	(void)wprintw(datawin,"n/a            ");
+        (void)snprintf(scr, sizeof(scr), "n/a");
+      (void)mvwprintw(datawin, 13, DATAWIN_VALUE_OFFSET + 5, "%-*s", 22, scr);
     }
 
   /* Be quiet if the user requests silence. */
@@ -439,18 +586,18 @@ static void update_gps_panel(struct gps_data_t *gpsdata,
   }
 }
 
-static void usage( char *prog) 
+static void usage( char *prog)
 {
-  (void)fprintf(stderr, 
-                "Usage: %s [-h] [-V] [-l {d|m|s}] [server[:port:[device]]]\n\n"
-                "  -h          Show this help, then exit\n"
-                "  -V          Show version, then exit\n"
-                "  -s          Be silent (don't print raw gpsd data)\n"
-                "  -l {d|m|s}  Select lat/lon format\n"
-                "                d = DD.dddddd\n"
-                "                m = DD MM.mmmm'\n"
-                "                s = DD MM' SS.sss\"\n"
-                , prog);
+  (void)fprintf(stderr,
+		"Usage: %s [-h] [-V] [-l {d|m|s}] [server[:port:[device]]]\n\n"
+		"  -h	  Show this help, then exit\n"
+		"  -V	  Show version, then exit\n"
+		"  -s	  Be silent (don't print raw gpsd data)\n"
+		"  -l {d|m|s}  Select lat/lon format\n"
+		"		d = DD.dddddd\n"
+		"		m = DD MM.mmmm'\n"
+		"		s = DD MM' SS.sss\"\n"
+		, prog);
 
   exit(1);
 }
@@ -462,8 +609,6 @@ int main(int argc, char *argv[])
   char *err_str = NULL;
   int c;
 
-  int xsize, ysize;
-
   struct timeval timeout;
   fd_set rfds;
   int data;
@@ -478,22 +623,22 @@ int main(int argc, char *argv[])
       fixclear_flag=1;
       break;
     case 'V':
-      (void)fprintf(stderr, "SVN ID: $Id: cgps.c 4074 2006-12-04 21:55:45Z jfrancis $ \n");
+      (void)fprintf(stderr, "SVN ID: $Id: cgps.c 4547 2007-12-13 01:38:22Z esr $ \n");
       exit(0);
     case 'l':
       switch ( optarg[0] ) {
       case 'd':
-        deg_type = deg_dd;
-        continue;
+	deg_type = deg_dd;
+	continue;
       case 'm':
-        deg_type = deg_ddmm;
-        continue;
+	deg_type = deg_ddmm;
+	continue;
       case 's':
-        deg_type = deg_ddmmss;
-        continue;
+	deg_type = deg_ddmmss;
+	continue;
       default:
-        (void)fprintf(stderr, "Unknown -l argument: %s\n", optarg);
-        /*@ -casebreak @*/
+	(void)fprintf(stderr, "Unknown -l argument: %s\n", optarg);
+	/*@ -casebreak @*/
       }
     case 'h': default:
       usage(argv[0]);
@@ -509,17 +654,17 @@ int main(int argc, char *argv[])
     server = arg;
     if (colon1 != NULL) {
       if (colon1 == arg)
-        server = NULL;
+	server = NULL;
       else
-        *colon1 = '\0';
+	*colon1 = '\0';
       port = colon1 + 1;
       colon2 = strchr(port, ':');
       if (colon2 != NULL) {
-        if (colon2 == port)
-          port = NULL;
-        else
-          *colon2 = '\0';
-        device = colon2 + 1;
+	if (colon2 == port)
+	  port = NULL;
+	else
+	  *colon2 = '\0';
+	device = colon2 + 1;
       }
     }
     colon1 = colon2 = NULL;
@@ -563,11 +708,11 @@ int main(int argc, char *argv[])
     case NL_NOSOCK:     err_str = "can't create socket"; break;
     case NL_NOSOCKOPT:  err_str = "error SETSOCKOPT SO_REUSEADDR"; break;
     case NL_NOCONNECT:  err_str = "can't connect to host"; break;
-    default:                    err_str = "Unknown"; break;
+    default:		    err_str = "Unknown"; break;
     }
-    (void)fprintf( stderr, 
-                   "cgps: no gpsd running or network error: %d, %s\n", 
-                   errno, err_str);
+    (void)fprintf( stderr,
+		   "cgps: no gpsd running or network error: %d, %s\n",
+		   errno, err_str);
     exit(2);
   }
 
@@ -614,173 +759,7 @@ int main(int argc, char *argv[])
   (void)signal(SIGINT,die);
   (void)signal(SIGHUP,die);
 
-  /* Set the window sizes per the following criteria:
-
-     1.  Set the window size to display the maximum number of
-     satellites possible, but not more than the size required to
-     display the maximum number of satellites gpsd is capable of
-     tracking (MAXCHANNELS - 2).
-
-     2.  If the screen size will not allow for the full complement of
-     satellites to be displayed, set the windows sizes smaller, but
-     not smaller than the number of lines necessary to display all of
-     the fields in the 'datawin'.  The list of displayed satellites
-     will be truncated to fit the available window size.  (TODO: If
-     the satellite list is truncated, omit the satellites not used to
-     obtain the current fix.)
-
-     3.  If the criteria in #2 is one line short of what is necessary
-     to display the 'datawin' data, drop the title at the top of the
-     screen and move everything up one line, also expanding the
-     satellite list by one.
-
-     4.  If the screen is large enough to display all possible
-     satellites (MAXCHANNELS - 2) with space still left at the bottom,
-     add a window at the bottom in which to scroll raw gpsd data.
-  */
-  (void)getmaxyx(stdscr,ysize,xsize);
-
-  if(compass_flag==1) {
-    if(ysize == MIN_COMPASS_DATAWIN_SIZE) {
-      title_flag = 0;
-      raw_flag = 0;
-      window_length = MIN_COMPASS_DATAWIN_SIZE;
-    } else if(ysize == MIN_COMPASS_DATAWIN_SIZE + 1) {
-      title_flag = 1;
-      raw_flag = 0;
-      window_length = MIN_COMPASS_DATAWIN_SIZE;
-    } else if(ysize >= MIN_COMPASS_DATAWIN_SIZE + 2) {
-      title_flag = 1;
-      raw_flag = 1;
-      window_length = MIN_COMPASS_DATAWIN_SIZE;
-    } else {
-      (void)mvprintw(0, 0, "Your screen must be at least 80x%d to run cgps.",MIN_COMPASS_DATAWIN_SIZE);
-      /*@ -nullpass @*/
-      (void)refresh();
-      /*@ +nullpass @*/
-      (void)sleep(5);
-      die(0);
-    }
-  } else {
-    if(ysize == MAX_SATWIN_SIZE) {
-      title_flag = 0;
-      raw_flag = 0;
-      window_length = MAX_SATWIN_SIZE;
-      display_sats = MAX_POSSIBLE_SATS;
-    } else if(ysize == MAX_SATWIN_SIZE + 1) {
-      title_flag = 0;
-      raw_flag = 1;
-      window_length = MAX_SATWIN_SIZE;
-      display_sats = MAX_POSSIBLE_SATS;
-    } else if(ysize > MAX_SATWIN_SIZE + 2) {
-      title_flag = 0;
-      raw_flag = 1;
-      window_length = MAX_SATWIN_SIZE;
-      display_sats = MAX_POSSIBLE_SATS;
-    } else if(ysize > MIN_GPS_DATAWIN_SIZE) {
-      title_flag = 1;
-      raw_flag = 0;
-      window_length = ysize - title_flag - raw_flag;
-      display_sats = window_length - SATWIN_OVERHEAD - title_flag - raw_flag;
-    } else if(ysize == MIN_GPS_DATAWIN_SIZE) {
-      title_flag = 0;
-      raw_flag = 0;
-      window_length = MIN_GPS_DATAWIN_SIZE;
-      display_sats = window_length - SATWIN_OVERHEAD - 1;
-    } else {
-      (void)mvprintw(0, 0, "Your screen must be at least 80x%d to run cgps.",MIN_GPS_DATAWIN_SIZE);
-      /*@ -nullpass @*/
-      (void)refresh();
-      /*@ +nullpass @*/
-      (void)sleep(5);
-      die(0);
-    }
-  }
-
-  /* Set up the screen for either a compass or a gps receiver. */
-  if(compass_flag==1) {
-    /* We're a compass, set up accordingly. */
-
-    /*@ -onlytrans @*/
-    datawin    = newwin(window_length, DATAWIN_WIDTH, title_flag, 0);
-    (void)nodelay(datawin,(bool)TRUE);
-    if(raw_flag==1) {
-      messages   = newwin(0, 0, title_flag + window_length, 0);
-
-      /*@ +onlytrans @*/
-      (void)scrollok(messages, true);
-      (void)wsetscrreg(messages, 0, ysize - (window_length + title_flag));
-    }
-
-    if(title_flag==1) {
-	(void)mvprintw(0, (int)(((DATAWIN_WIDTH + SATELLITES_WIDTH) / 2) - (strlen(TITLE) / 2)), TITLE);
-    }
-
-    /*@ -nullpass @*/
-    (void)refresh();
-    /*@ +nullpass @*/
-    
-    /* Do the initial field label setup. */
-    (void)mvwprintw(datawin, 1, DATAWIN_DESC_OFFSET, "Time:");
-    (void)mvwprintw(datawin, 2, DATAWIN_DESC_OFFSET, "Heading:");
-    (void)mvwprintw(datawin, 3, DATAWIN_DESC_OFFSET, "Pitch:");
-    (void)mvwprintw(datawin, 4, DATAWIN_DESC_OFFSET, "Roll:");
-    (void)mvwprintw(datawin, 5, DATAWIN_DESC_OFFSET, "Dip:");
-    (void)mvwprintw(datawin, 6, DATAWIN_DESC_OFFSET, "Rcvr Type:");
-    (void)wborder(datawin, 0, 0, 0, 0, 0, 0, 0, 0);
-
-  } else {
-    /* We're a GPS, set up accordingly. */
-
-    /*@ -onlytrans @*/
-    datawin    = newwin(window_length, DATAWIN_WIDTH, title_flag, 0);
-    satellites = newwin(window_length, SATELLITES_WIDTH, title_flag, DATAWIN_WIDTH);
-    (void)nodelay(datawin,(bool)TRUE);
-    if(raw_flag==1) {
-      messages   = newwin(ysize - (window_length + title_flag), xsize, title_flag + window_length, 0);
-
-      /*@ +onlytrans @*/
-      (void)scrollok(messages, true);
-      (void)wsetscrreg(messages, 0, ysize - (window_length + title_flag));
-    }
-
-    if(title_flag==1) {
-	(void)mvprintw(0, (int)(((DATAWIN_WIDTH + SATELLITES_WIDTH) / 2) - (strlen(TITLE) / 2)), TITLE);
-    }
-    /*@ -nullpass @*/
-    (void)refresh();
-    /*@ +nullpass @*/
-    
-    /* Do the initial field label setup. */
-    (void)mvwprintw(datawin, 1, DATAWIN_DESC_OFFSET, "Time:");
-    (void)mvwprintw(datawin, 2, DATAWIN_DESC_OFFSET, "Latitude:");
-    (void)mvwprintw(datawin, 3, DATAWIN_DESC_OFFSET, "Longitude:");
-    (void)mvwprintw(datawin, 4, DATAWIN_DESC_OFFSET, "Altitude:");
-    (void)mvwprintw(datawin, 5, DATAWIN_DESC_OFFSET, "Speed:");
-    (void)mvwprintw(datawin, 6, DATAWIN_DESC_OFFSET, "Heading:");
-    (void)mvwprintw(datawin, 7, DATAWIN_DESC_OFFSET, "Climb:");
-    (void)mvwprintw(datawin, 8, DATAWIN_DESC_OFFSET, "Status:");
-    (void)mvwprintw(datawin, 9, DATAWIN_DESC_OFFSET, "GPS Type:");
-
-    /* Note that the following four fields are exceptions to the
-       sizing rule.  The minimum window size does not include these
-       fields, if the window is too small, they get excluded.  This
-       may or may not change if/when the output for these fields is
-       fixed and/or people request their permanance.  They're only
-       there in the first place because I arbitrarily thought they
-       sounded interesting. ;^) */
-
-    if(window_length >= (MIN_GPS_DATAWIN_SIZE + 4)) {
-      (void)mvwprintw(datawin, 10, DATAWIN_DESC_OFFSET, "Horizontal Err:");
-      (void)mvwprintw(datawin, 11, DATAWIN_DESC_OFFSET, "Vertical Err:");
-      (void)mvwprintw(datawin, 12, DATAWIN_DESC_OFFSET, "Course Err:");
-      (void)mvwprintw(datawin, 13, DATAWIN_DESC_OFFSET, "Speed Err:");
-    }
-
-    (void)wborder(datawin, 0, 0, 0, 0, 0, 0, 0, 0);
-    (void)mvwprintw(satellites, 1,1, "PRN:   Elev:  Azim:  SNR:  Used:");
-    (void)wborder(satellites, 0, 0, 0, 0, 0, 0, 0, 0);
-  }
+  windowsetup();
 
   /* Here's where updates go now that things are established. */
   if(compass_flag==1) {
@@ -794,7 +773,7 @@ int main(int argc, char *argv[])
 
   /* heart of the client */
   for (;;) {
-    
+
     /* watch to see when it has input */
     FD_ZERO(&rfds);
     FD_SET(gpsdata->gps_fd, &rfds);
@@ -805,7 +784,7 @@ int main(int argc, char *argv[])
 
     /* check if we have new information */
     data = select(gpsdata->gps_fd + 1, &rfds, NULL, NULL, &timeout);
-        
+
     if (data == -1) {
       fprintf( stderr, "cgps: socket error\n");
       exit(2);
@@ -813,7 +792,7 @@ int main(int argc, char *argv[])
     else if( data ) {
       /* code that calls gps_poll(gpsdata) */
       if (gps_poll(gpsdata) != 0)
-        die(1);
+	die(1);
     }
 
     /* Check for user input.
@@ -822,7 +801,7 @@ int main(int argc, char *argv[])
      makes no sense for a True North compass).  No doubt there will be
      other examples as cgps grows more bells and whistles. */
     c=wgetch(datawin);
-        
+
     switch ( c ) {
       /* Quit */
     case 'q':
@@ -832,20 +811,20 @@ int main(int argc, char *argv[])
       /* Toggle spewage of raw gpsd data. */
     case 's':
       if(silent_flag==0) {
-        silent_flag=1;
+	silent_flag=1;
       } else {
-        silent_flag=0;
+	silent_flag=0;
       }
       break;
 
       /* Toggle fix clear. */
     case 'j':
       if(fixclear_flag==0) {
-        fixclear_flag=1;
-        (void)gps_query(gpsdata, "j=1\n");
+	fixclear_flag=1;
+	(void)gps_query(gpsdata, "j=1\n");
       } else {
-        fixclear_flag=0;
-        (void)gps_query(gpsdata, "j=0\n");
+	fixclear_flag=0;
+	(void)gps_query(gpsdata, "j=0\n");
       }
       break;
 
@@ -859,5 +838,5 @@ int main(int argc, char *argv[])
     }
 
   }
- 
+
 }
