@@ -1,4 +1,4 @@
-/* $Id: driver_proto.c 4636 2007-12-27 07:21:29Z ckuethe $
+/* $Id: driver_proto.c 4661 2008-01-19 22:54:23Z garyemiller $
  *
  * A prototype driver.  Doesn't run, doesn't even compile.
  *
@@ -366,7 +366,7 @@ static void proto_wrapup(struct gps_device_t *session)
 /* This is everything we export */
 struct gps_type_t proto_binary = {
     /* Full name of type */
-    .typename         = "Prototype driver",
+    .type_name        = "Prototype driver",
     /* Response string that identifies device (not active) */
     .trigger          = NULL,
     /* Number of satellite channels supported by the device */
