@@ -1,4 +1,4 @@
-/* $Id: drivers.c 4651 2008-01-01 14:53:16Z esr $ */
+/* $Id: drivers.c 4663 2008-01-20 23:35:32Z garyemiller $ */
 #include <sys/types.h>
 #include <sys/ioctl.h>
 #include <sys/time.h>
@@ -95,7 +95,7 @@ gps_mask_t nmea_parse_input(struct gps_device_t *session)
 
 		if (trigger!=NULL && strncmp((char *)session->packet.outbuffer, trigger, strlen(trigger))==0 && isatty(session->gpsdata.gps_fd)!=0) {
 		    gpsd_report(LOG_PROG, "found %s.\n", trigger);
-		    (void)gpsd_switch_driver(session, (*dp)->typename);
+		    (void)gpsd_switch_driver(session, (*dp)->type_name);
 		    return 1;
 		}
 	    }
@@ -235,7 +235,7 @@ static void nmea_probe_subtype(struct gps_device_t *session, unsigned int seq)
 }
 
 static struct gps_type_t nmea = {
-    .typename       = "Generic NMEA",	/* full name of type */
+    .type_name      = "Generic NMEA",	/* full name of type */
     .trigger	    = NULL,		/* it's the default */
     .channels       = 12,		/* consumer-grade GPS */
     .probe_wakeup   = NULL,		/* no wakeup to be done before hunt */
@@ -258,7 +258,7 @@ static struct gps_type_t nmea = {
     .cycle	    = 1,		/* updates every second */
 };
 
-#ifdef GARMIN_ENABLE
+#if defined(GARMIN_ENABLE) && defined(NMEA_ENABLE)
 /**************************************************************************
  *
  * Garmin NMEA
@@ -268,15 +268,18 @@ static struct gps_type_t nmea = {
 #ifdef ALLOW_RECONFIGURE
 static void garmin_nmea_configurator(struct gps_device_t *session, unsigned int seq)
 {
-#if defined(NMEA_ENABLE)
     /*
      * Receivers like the Garmin GPS-10 don't handle having having a lot of
      * probes shoved at them very well.
      */
     switch (seq) {
     case 0:
-	/* reset some config, AutoFix, WGS84, PPS */
-	(void)nmea_send(session->gpsdata.gps_fd, "$PGRMC,A,,100,,,,,,A,,1,2,4,30");
+	/* reset some config, AutoFix, WGS84, PPS 
+	 * Set the PPS pulse length to 40ms which leaves the Garmin 18-5hz 
+         * with a 160ms low state.
+         * NOTE: new PPS only takes effect after next power cycle
+         */
+	(void)nmea_send(session->gpsdata.gps_fd, "$PGRMC,A,,100,,,,,,A,,1,2,1,30");
 	break;
     case 1:
 	/* once a sec, no averaging, NMEA 2.3, WAAS */
@@ -306,11 +309,10 @@ static void garmin_nmea_configurator(struct gps_device_t *session, unsigned int
 	break;
     }
 }
-#endif /* NMEA_ENABLE */
 #endif /* ALLOW_RECONFIGURE */
 
 static struct gps_type_t garmin = {
-    .typename       = "Garmin Serial",	/* full name of type */
+    .type_name      = "Garmin Serial",	/* full name of type */
     .trigger	    = "$PGRMC,",	/* Garmin private */
     .channels       = 12,		/* not used by this driver */
     .probe_wakeup   = NULL,		/* no wakeup to be done before hunt */
@@ -332,7 +334,7 @@ static struct gps_type_t garmin = {
     .wrapup	    = NULL,		/* no wrapup */
     .cycle	    = 1,		/* updates every second */
 };
-#endif /* GARMIN_ENABLE */
+#endif /* GARMIN_ENABLE && NMEA_ENABLE */
 
 #ifdef ASHTECH_ENABLE
 /**************************************************************************
@@ -371,7 +373,7 @@ static void ashtech_ping(struct gps_device_t *session)
 }
 
 static struct gps_type_t ashtech = {
-    .typename       = "Ashtech",	/* full name of type */
+    .type_name      = "Ashtech",	/* full name of type */
     .trigger	    = "$PASHR,RID,",	/* Ashtech receivers respond thus */
     .channels       = 24,		/* not used, GG24 has 24 channels */
     .probe_wakeup   = ashtech_ping,	/* no wakeup to be done before hunt */
@@ -416,7 +418,7 @@ static void fv18_configure(struct gps_device_t *session, unsigned int seq)
 #endif /* ALLOW_RECONFIGURE */
 
 static struct gps_type_t fv18 = {
-    .typename       = "San Jose Navigation FV18",	/* full name of type */
+    .type_name      = "San Jose Navigation FV18",	/* full name of type */
     .trigger	    = "$PFEC,GPint,",	/* FV18s should echo the probe */
     .channels       = 12,		/* not used by this driver */
     .probe_wakeup   = NULL,		/* no wakeup to be done before hunt */
@@ -464,7 +466,7 @@ static void gpsclock_probe_subtype(struct gps_device_t *session, unsigned int se
 }
 
 static struct gps_type_t gpsclock = {
-    .typename       = "Furuno Electric GH-79L4",	/* full name of type */
+    .type_name      = "Furuno Electric GH-79L4",	/* full name of type */
     .trigger	    = "$PFEC,GPssd",	/* GPSclock should echo the probe */
     .channels       = 12,		/* not used by this driver */
     .probe_wakeup   = NULL,		/* no wakeup to be done before hunt */
@@ -520,7 +522,7 @@ static void tripmate_configurator(struct gps_device_t *session, unsigned int seq
 #endif /* ALLOW_RECONFIGURE */
 
 static struct gps_type_t tripmate = {
-    .typename      = "Delorme TripMate",	/* full name of type */
+    .type_name     = "Delorme TripMate",	/* full name of type */
     .trigger       ="ASTRAL",			/* tells us to switch */
     .channels      = 12,			/* consumer-grade GPS */
     .probe_wakeup  = NULL,			/* no wakeup before hunt */
@@ -578,7 +580,7 @@ static void earthmate_probe_subtype(struct gps_device_t *session, unsigned int s
 
 /*@ -redef @*/
 static struct gps_type_t earthmate = {
-    .typename      = "Delorme EarthMate (pre-2003, Zodiac chipset)",
+    .type_name     = "Delorme EarthMate (pre-2003, Zodiac chipset)",
     .trigger       = "EARTHA",			/* Earthmate trigger string */
     .channels      = 12,			/* not used by NMEA parser */
     .probe_wakeup  = NULL,			/* no wakeup to be done before hunt */
@@ -751,7 +753,7 @@ static bool tnt_probe(struct gps_device_t *session)
 }
 
 struct gps_type_t trueNorth = {
-    .typename       = "True North",	/* full name of type */
+    .type_name      = "True North",	/* full name of type */
     .trigger	    = " TNT1500",
     .channels       = 0,		/* not an actual GPS at all */
     .probe_wakeup   = NULL,		/* this will become a real method */
@@ -792,7 +794,7 @@ static gps_mask_t rtcm104_analyze(struct gps_device_t *session)
 }
 
 static struct gps_type_t rtcm104 = {
-    .typename      = "RTCM104",		/* full name of type */
+    .type_name     = "RTCM104",		/* full name of type */
     .trigger       = NULL,		/* no recognition string */
     .channels      = 0,			/* not used */
     .probe_wakeup  = NULL,		/* no wakeup to be done before hunt */
@@ -831,7 +833,7 @@ static gps_mask_t garmintxt_parse_input(struct gps_device_t *session)
 
 
 static struct gps_type_t garmintxt = {
-    .typename      = "Garmin Simple Text",		/* full name of type */
+    .type_name     = "Garmin Simple Text",		/* full name of type */
     .trigger       = NULL,		/* no recognition string */
     .channels      = 0,			/* not used */
     .probe_wakeup  = NULL,		/* no wakeup to be done before hunt */
