@@ -1,4 +1,4 @@
-/* $Id: evermore.c 4373 2007-06-01 22:30:43Z esr $ */
+/* $Id: evermore.c 4661 2008-01-19 22:54:23Z garyemiller $ */
 /*
  * This is the gpsd driver for EverMore GPSes operating in binary mode.
  * About the only thing this gives us that NMEA won't is TDOP.
@@ -483,7 +483,7 @@ static void evermore_wrap(struct gps_device_t *session)
 /* this is everything we export */
 struct gps_type_t evermore_binary =
 {
-    .typename       = "EverMore binary",	/* full name of type */
+    .type_name      = "EverMore binary",	/* full name of type */
     .trigger        = "$PEMT,", 		/* recognize the type */
     .channels       = EVERMORE_CHANNELS,	/* consumer-grade GPS */
     .probe_wakeup   = NULL,			/* no wakeup to be done before hunt */
