@@ -1,4 +1,4 @@
-/* $Id: geoid.c 3666 2006-10-26 23:11:51Z ckuethe $ */
+/* $Id: geoid.c 4257 2007-01-18 03:56:43Z ckuethe $ */
 /* 
  * geoid.c -- ECEF to WGS84 conversions, including ellipsoid-to-MSL height
  *
@@ -11,6 +11,8 @@
 #include "gpsd_config.h"
 #include "gpsd.h"
 
+static double fix_minuz(double d);
+
 static double bilinear(double x1, double y1, double x2, double y2, double x, double y, double z11, double z12, double z21, double z22)
 {
  double delta;
@@ -58,6 +60,14 @@ double wgs84_separation(double lat, double lon)
     ilat=(int)floor(( 90.+lat)/10);
     ilon=(int)floor((180.+lon)/10);
 	
+    /* sanity checks to prevent segfault on bad data */
+    if ( ( ilat > 90 ) || ( ilat < -90 ) ) {
+	return 0.0;
+    }
+    if ( ( ilon > 180 ) || ( ilon < -180 ) ) {
+	return 0.0;
+    }
+
     ilat1=ilat;
     ilon1=ilon;
     ilat2=(ilat < GEOID_ROW-1)? ilat+1:ilat;
@@ -104,13 +114,28 @@ void ecef_to_wgs84fix(struct gps_data_t *gpsdata,
     veast = -vx*sin(lambda)+vy*cos(lambda);
     gpsdata->fix.climb = vx*cos(phi)*cos(lambda)+vy*cos(phi)*sin(lambda)+vz*sin(phi);
     gpsdata->fix.speed = sqrt(pow(vnorth,2) + pow(veast,2));
-    heading = atan2(veast,vnorth);
+    heading = atan2(fix_minuz(veast), fix_minuz(vnorth));
     /*@ +evalorder @*/
     if (heading < 0)
 	heading += 2 * PI;
     gpsdata->fix.track = heading * RAD_2_DEG;
 }
 
+/*
+ * Some systems propagate the sign along with zero. This messes up
+ * certain trig functions, like atan2():
+ *    atan2(+0, +0) = 0
+ *    atan2(+0, -0) = PI
+ * Obviously that will break things. Luckily the "==" operator thinks
+ * that -0 == +0; we will use this to return an unambiguous value.
+ *
+ * I hereby decree that zero is not allowed to have a negative sign!
+ */
+static double fix_minuz(double d)
+{
+    return ((d == 0.0) ? 0.0 : d);
+}
+
 #ifdef TESTMAIN
 #include <stdio.h>
 #include <stdlib.h>
