@@ -1,4 +1,4 @@
-/* $Id: gpsd.c 4640 2007-12-27 18:53:52Z esr $ */
+/* $Id: gpsd.c 4661 2008-01-19 22:54:23Z garyemiller $ */
 #include <sys/types.h>
 #include <sys/socket.h>
 #include <sys/un.h>
@@ -216,7 +216,7 @@ in which case it specifies an input source for DGPS or ntrip data.\n\
 The following driver types are compiled into this gpsd instance:\n",
 	   DEFAULT_GPSD_PORT);
     for (dp = gpsd_drivers; *dp; dp++) {
-	(void)printf("    %s\n", (*dp)->typename);
+	(void)printf("    %s\n", (*dp)->type_name);
     }
 }
 
@@ -1650,7 +1650,7 @@ int main(int argc, char *argv[])
 			char id[NMEA_MAX];
 			assert(channel->device_type != NULL);
 			(void)snprintf(id, sizeof(id), "GPSD,I=%s",
-				       channel->device_type->typename);
+				       channel->device_type->type_name);
 			if (channel->subtype[0] != '\0') {
 			    (void)strlcat(id, " ", sizeof(id));
 			    (void)strlcat(id,channel->subtype,sizeof(id));
