@@ -17,21 +17,21 @@
 
 /* Feature configuration switches begin here */
 #define PACKAGE_VERSION ""
-#define VERSION "2.36"
+#define VERSION "2.37"
 #define ASHTECH_ENABLE 1
 /* #undef DBUS_ENABLE */
 #define EARTHMATE_ENABLE 1
 #define EVERMORE_ENABLE 1
-#define FV18_ENABLE 1
-#define GARMINTXT_ENABLE 1
+/* #undef FV18_ENABLE */
+/* #undef GARMINTXT_ENABLE */
 #define GARMIN_ENABLE 1
 #define GPSCLOCK_ENABLE 1
 #define ITRAX_ENABLE 1
 #define NAVCOM_ENABLE 1
 #define NMEA_ENABLE 1
-#define NTPSHM_ENABLE 1
-#define NTRIP_ENABLE 1
-#define PPS_ENABLE 1
+/* #undef NTPSHM_ENABLE */
+/* #undef NTRIP_ENABLE */
+/* #undef PPS_ENABLE */
 #define RTCM104_ENABLE 1
 #define SIRF_ENABLE 1
 /* #undef SQUELCH_ENABLE */
@@ -41,7 +41,7 @@
 #define UBX_ENABLE 1
 /* Feature configuration switches end here */
 
-/* $Id: gpsd.h-tail 4573 2007-12-18 02:23:16Z esr $ */
+/* $Id: gpsd.h-tail 4665 2008-01-21 00:43:01Z garyemiller $ */
 
 /* Some internal capabilities depend on which drivers we're compiling. */
 #ifdef EARTHMATE_ENABLE
@@ -214,7 +214,7 @@ struct gps_device_t;
 
 struct gps_type_t {
 /* GPS method table, describes how to talk to a particular GPS type */
-    /*@observer@*/char *typename;
+    /*@observer@*/char *type_name;
     /*@observer@*//*@null@*/char *trigger;
     int channels;
     /*@null@*/bool (*probe_detect)(struct gps_device_t *session);
