@@ -1,12 +1,11 @@
-/* $Id: gpsd_dbus.h 3874 2006-11-13 05:17:19Z esr $ */
-#include "gpsd_config.h"
+/* $Id: gpsd_dbus.h 4543 2007-12-12 21:34:19Z esr $ */
 #ifdef DBUS_ENABLE
 
 #ifndef _gpsd_dbus_h_
 #define _gpsd_dbus_h_
 
-//#define DBUS_API_SUBJECT_TO_CHANGE
 #include <dbus/dbus.h>
+
 #include "gpsd.h"
 
 int initialize_dbus_connection (void);
