@@ -1,5 +1,5 @@
 """
-$Id: gpsfake.py 4121 2006-12-12 17:30:01Z esr $
+$Id: gpsfake.py 4646 2007-12-28 04:45:02Z ckuethe $
 
 gpsfake.py -- classes for creating a controlled test environment around gpsd.
 
@@ -67,19 +67,29 @@ To allow for adding and removing clients while the test is running,
 run in threaded mode by calling the start() method.  This simply calls
 the run method in a subthread, with locking of critical regions.
 """
-import sys, os, time, signal, pty, termios
-import string, exceptions, threading, socket, commands
+import sys, os, time, signal, pty, termios # fcntl, array, struct
+import exceptions, threading, socket
 import gps
 
+# Define a per-character delay on writes so we won't spam the buffers
+# in the pty layer or gpsd itself.  The magic number here has to be
+# derived from observation.  If it's too high you'll slow the tests
+# down a lot.  If it's too low you'll get random spurious regression
+# failures that usually look like lines missing from the end of the
+# test output relative to the check file.  This number might have to
+# be adusted upward on faster machines.
+WRITE_PAD = 15.0
+
 class TestLoadError(exceptions.Exception):
     def __init__(self, msg):
         self.msg = msg
 
 class TestLoad:
     "Digest a logfile into a list of sentences we can cycle through."
-    def __init__(self, logfp):
+    def __init__(self, logfp, predump=False):
         self.sentences = []	# This and .packtype are the interesting bits
         self.logfp = logfp
+        self.predump = predump
         self.logfile = logfp.name
         self.type = None
         self.serial = None
@@ -109,7 +119,7 @@ class TestLoad:
                             stopbits = int(params[2])
                         else:
                             raise ValueError
-                    except ValueError, IndexError:
+                    except (ValueError, IndexError):
                         raise TestLoadError("bad serial-parameter spec in %s"%\
                                             logfp.name)
                     
@@ -119,8 +129,9 @@ class TestLoad:
         # Grab the packets
         while True:
             packet = self.packet_get()
-            #print "I see: %s, length %d" % (`packet`, len(packet))
-            if not packet:
+            if self.predump:
+                print `packet`
+            if not packet or packet == "\n":
                 break
             else:
                 self.sentences.append(packet)
@@ -140,11 +151,25 @@ class TestLoad:
             self.legend = "gpsfake: packet %d: "
             self.idoffset = 3
             self.textual = False
+        elif self.sentences[0][0] == '\x02':
+            self.packtype = "Navcom binary"
+            self.legend = "gpsfake: packet %d"
+            self.textual = False
         elif self.sentences[0][0] == '\x10':
             self.packtype = "TSIP binary"
             self.legend = "gpsfake: packet %d: "
             self.idoffset = 1
             self.textual = False
+        elif self.sentences[0][0] == '\xb5':
+            self.packtype = "uBlox"
+            self.legend = "gpsfake: packet %d: "
+            self.idoffset = None
+            self.textual = False
+        elif self.sentences[0][0] == '\x3c':
+            self.packtype = "iTrax"
+            self.legend = "gpsfake: packet %d: "
+            self.idoffset = None
+            self.textual = False
         elif self.type == "RTCM":
             self.packtype = "RTCM"
             self.legend = None
@@ -177,9 +202,17 @@ class TestLoad:
             fourth = self.logfp.read(1)
             fifth = self.logfp.read(1)
             sixth = self.logfp.read(1)
-            id = ord(third) | (ord(fourth) << 8)
+            #id = ord(third) | (ord(fourth) << 8)
             ndata = ord(fifth) | (ord(sixth) << 8)
             return "\xff\x81" + third + fourth + fifth + sixth + self.logfp.read(2*ndata+6)
+        elif first == '\x02' and second == '\x99':		# Navcom
+            third = self.logfp.read(1)
+	    fourth = self.logfp.read(1)
+            fifth = self.logfp.read(1)
+            sixth = self.logfp.read(1)
+            #id = ord(fourth)
+            ndata = ord(fifth) | (ord(sixth) << 8)
+            return "\x02\x99\x66" + fourth + fifth + sixth + self.logfp.read(ndata-2)
         elif first == '\x10':					# TSIP
             packet = first + second
             delcnt = 0
@@ -196,8 +229,31 @@ class TestLoad:
                 else:
                     delcnt = 0
             return packet
+        elif first == '\xb5' and second == '\x62':		# ubx
+            third = self.logfp.read(1)
+            fourth = self.logfp.read(1)
+            fifth = self.logfp.read(1)
+            sixth = self.logfp.read(1)
+            # classid = third
+            # messageid = fourth
+            ndata = ord(fifth) | (ord(sixth) << 8)
+            return "\xb5\x62" + third + fourth + fifth + sixth + self.logfp.read(ndata+2)
+        elif first == '\x3c' and second == '\x21':		# italk
+            third = self.logfp.read(1)
+            fourth = self.logfp.read(1)
+            fifth = self.logfp.read(1)
+            sixth = self.logfp.read(1)
+            seventh = self.logfp.read(1)
+            # srcnode = third
+            # dstnode = fourth
+            # messageid = fifth
+            # transaction = sixth
+            ndata = (ord(seventh)+1)*2 + 1
+            return "\x3c\x21" + third + fourth + fifth + sixth + seventh + self.logfp.read(ndata)
+        elif first == "\n":	# Use this to ignore trailing EOF on logs
+            return "\n"
         else:
-            raise PacketError("unknown packet type, leader %s, (0x%x)" % (first, ord(first)))
+            raise PacketError("unknown packet type, leader %s (0x%x)" % (`first`, ord(first)))
 
 class PacketError(exceptions.Exception):
     def __init__(self, msg):
@@ -207,7 +263,7 @@ class FakeGPS:
     "A fake GPS is a pty with a test log ready to be cycled to it."
     def __init__(self, logfp,
                  speed=4800, databits=8, parity='N', stopbits=1,
-                 verbose=False):
+                 verbose=False, predump=False):
         self.verbose = verbose
         self.go_predicate = lambda: True
         self.readers = 0
@@ -237,7 +293,7 @@ class FakeGPS:
         speed = baudrates[speed]	# Throw an error if the speed isn't legal
         if type(logfp) == type(""):
             logfp = open(logfp, "r");            
-        self.testload = TestLoad(logfp)
+        self.testload = TestLoad(logfp, predump)
         # FIXME: explicit arguments should probably override this
         #if self.testload.serial:
         #    (speed, databits, parity, stopbits) = self.testload.serial
@@ -268,19 +324,29 @@ class FakeGPS:
                           [iflag, oflag, cflag, lflag, ispeed, ospeed, cc])
     def read(self):
         "Discard control strings written by gpsd."
+        # A tcflush implementation works on Linux but fails on OpenBSD 4.
         termios.tcflush(self.master_fd, termios.TCIFLUSH)
+        # Alas, the FIONREAD version also works on Linux and fails on OpenBSD.
+        #try:
+        #    buf = array.array('i', [0])
+        #    fcntl.ioctl(self.master_fd, termios.FIONREAD, buf, True)
+        #    n = struct.unpack('i', buf)[0]
+        #    os.read(self.master_fd, n)
+        #except IOError:
+        #    pass
+
     def feed(self):
         "Feed a line from the contents of the GPS log to the daemon."
         line = self.testload.sentences[self.index % len(self.testload.sentences)]
         os.write(self.master_fd, line)
-        # Delay so we won't spam the buffers in the pty layer or gpsd itself.
-        # Assumptions: 1 character is 10 bits (generous; at 8N1 it will be 9).
-        time.sleep((10.0 * len(line)) / self.speed)
+        time.sleep((WRITE_PAD * len(line)) / self.speed)
         self.index += 1
 
 class DaemonError(exceptions.Exception):
     def __init__(self, msg):
         self.msg = msg
+    def __str__(self):
+        return repr(self.msg)
 
 class DaemonInstance:
     "Control a gpsd instance."
@@ -294,7 +360,21 @@ class DaemonInstance:
         self.pidfile  = "/tmp/gpsfake_pid-%s" % os.getpid()
     def spawn(self, options, port, background=False, prefix=""):
         "Spawn a daemon instance."
-        self.spawncmd = "gpsd -N -S %s -F %s -P %s %s" % (port, self.control_socket, self.pidfile, options)
+        self.spawncmd = None
+        if not '/usr/sbin' in os.environ['PATH']:
+            os.environ['PATH']=os.environ['PATH'] + ":/usr/sbin"
+        for path in os.environ['PATH'].split(':'):
+            _spawncmd = "%s/gpsd" % path
+            if os.path.isfile(_spawncmd) and os.access(_spawncmd, os.X_OK):
+                self.spawncmd = _spawncmd
+                break
+
+        if not self.spawncmd:
+            raise DaemonError("Cannot execute gpsd: executable not found.")
+        # The -b option to suppress hanging on probe returns is needed to cope
+        # with OpenBSD (and possibly other non-Linux systems) that don't support
+        # anything we can use to implement the FakeGPS.read() method
+        self.spawncmd += " -b -N -S %s -F %s -P %s %s" % (port, self.control_socket, self.pidfile, options)
         if prefix:
             self.spawncmd = prefix + " " + self.spawncmd.strip()
         if background:
@@ -334,7 +414,7 @@ class DaemonInstance:
     def is_alive(self):
         "Is the daemon still alive?"
         try:
-            st = os.kill(self.pid, 0)
+            os.kill(self.pid, 0)
             return True
         except OSError:
             return False
@@ -367,9 +447,10 @@ class TestSessionError(exceptions.Exception):
 class TestSession:
     "Manage a session including a daemon with fake GPSes and clients."
     CLOSE_DELAY = 1
-    def __init__(self, prefix=None, port=None, options=None, verbose=False):
+    def __init__(self, prefix=None, port=None, options=None, verbose=False, predump=False):
         "Initialize the test session by launching the daemon."
         self.verbose = verbose
+        self.predump = predump
         self.daemon = DaemonInstance()
         self.fakegpslist = {}
         self.client_id = 0
@@ -397,7 +478,7 @@ class TestSession:
         "Add a simulated GPS being fed by the specified logfile."
         self.progress("gpsfake: gps_add(%s, %d)\n" % (logfile, speed))
         if logfile not in self.fakegpslist:
-            newgps = FakeGPS(logfile, speed=speed, verbose=self.verbose)
+            newgps = FakeGPS(logfile, speed=speed, verbose=self.verbose, predump=self.predump)
             if pred:
                 newgps.go_predicate = pred
             elif self.default_predicate:
@@ -422,22 +503,22 @@ class TestSession:
         self.client_id += 1
         self.progress("gpsfake: client %d has %s\n" % (self.client_id,newclient.device))
         if commands:
-            self.initialize(newclient, commands), 
-        return newclient.id
+            self.initialize(newclient, commands) 
+        return self.client_id
     def client_query(self, id, commands):
         "Ship a command to a client channel, get a response (threaded mode only)."
         self.progress("gpsfake: client_query(%d, %s)\n" % (id, `commands`))
-        for object in self.runqueue:
-            if isinstance(object, gps.gps) and object.id == id:
-                client.query(commands)
-                return client.response
+        for obj in self.runqueue:
+            if isinstance(obj, gps.gps) and obj.id == id:
+                obj.query(commands)
+                return obj.response
         return None
-    def client_remove(self, id):
+    def client_remove(self, cid):
         "Terminate a client session."
-        self.progress("gpsfake: client_remove(%d)\n" % id)
-        for object in self.runqueue:
-            if isinstance(object, gps.gps) and object.id == id:
-                self.remove(object)
+        self.progress("gpsfake: client_remove(%d)\n" % cid)
+        for obj in self.runqueue:
+            if isinstance(obj, gps.gps) and obj.id == cid:
+                self.remove(obj)
                 return True
         else:
             return False
