@@ -1,11 +1,27 @@
-/* $Id: gpsflash.c 4078 2006-12-05 05:00:54Z ckuethe $ */
+/* $Id: gpsflash.c 4471 2007-12-08 08:02:57Z ckuethe $ */
 /*
  * This is the GPS-type-independent part of the gpsflash program.
  *
- * Copyright (c) 2005 Chris Kuethe <chris.kuethe@gmail.com>
+ */
+/*
+ * Copyright (c) 2005-2007 Chris Kuethe <chris.kuethe@gmail.com>
+ * Copyright (c) 2005-2007 Eric S. Raymond <esr@thyrsus.com>
+ *
+ * Permission to use, copy, modify, and distribute this software for any
+ * purpose with or without fee is hereby granted, provided that the above
+ * copyright notice and this permission notice appear in all copies.
+ *
+ * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
+ * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
+ * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
+ * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
+ * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
+ * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
+ * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  */
 #include <sys/types.h>
 #include <stdarg.h>
+#include <string.h>
 #include "gpsd_config.h"
 #include "gpsd.h"
 #include "gpsflash.h"
@@ -343,7 +359,7 @@ main(int argc, char **argv){
 			verbosity = atoi(optarg);
 			break;
 		case 'V':
-			(void)fprintf(stderr, "SVN ID: $Id: gpsflash.c 4078 2006-12-05 05:00:54Z ckuethe $ \n");
+			(void)fprintf(stderr, "SVN ID: $Id: gpsflash.c 4471 2007-12-08 08:02:57Z ckuethe $ \n");
 			exit(0);
 		default:
 			usage();
