@@ -1,4 +1,4 @@
-/* $Id: gpspipe.c 4125 2006-12-12 22:50:23Z ckuethe $ */
+/* $Id: gpspipe.c 4315 2007-03-24 22:03:17Z ckuethe $ */
 /*
  * gpspipe
  *
@@ -81,8 +81,8 @@ static void open_serial(char* device)
 
 static void usage(void)
 {
-    (void)fprintf(stderr, "Usage: gpspipe [OPTIONS] [server[:port]]\n\n"
-		  "SVN ID: $Id: gpspipe.c 4125 2006-12-12 22:50:23Z ckuethe $ \n"
+    (void)fprintf(stderr, "Usage: gpspipe [OPTIONS] [server[:port[:device]]]\n\n"
+		  "SVN ID: $Id: gpspipe.c 4315 2007-03-24 22:03:17Z ckuethe $ \n"
 		  "-h Show this help.\n"
 		  "-r Dump raw NMEA.\n"
 		  "-R Dump super-raw mode (GPS binary).\n"
@@ -137,7 +137,7 @@ int main( int argc, char **argv)
 	    (void)strlcat(buf, "j=1;", sizeof(buf));
 	    break;
 	case 'V':
-	    (void)fprintf(stderr, "%s: SVN ID: $Id: gpspipe.c 4125 2006-12-12 22:50:23Z ckuethe $ \n", argv[0]);
+	    (void)fprintf(stderr, "%s: SVN ID: $Id: gpspipe.c 4315 2007-03-24 22:03:17Z ckuethe $ \n", argv[0]);
 	    exit(0);
 	case 's':
 	    serialport = optarg;
@@ -180,6 +180,7 @@ int main( int argc, char **argv)
 		    *colon2 = '\0';
 		}
 		device = colon2 + 1;
+		(void)snprintf(buf, sizeof(buf), "%sF=%s\n", buf, device);
 	    }
 	}
 	colon1 = colon2 = NULL;
@@ -208,6 +209,9 @@ int main( int argc, char **argv)
 	exit(1);
     }
 
+    if (isatty(STDERR_FILENO) == 0)
+	vflag = 0;
+
     for(;;) {
 	int i = 0;
 	int j = 0;
