@@ -1,4 +1,4 @@
-/* $Id: gpsutils.c 4108 2006-12-08 11:48:31Z esr $ */
+/* $Id: gpsutils.c 4652 2008-01-01 16:47:10Z ckuethe $ */
 /* gpsutils.c -- code shared between low-level and high-level interfaces */
 #include <sys/types.h>
 #include <stdio.h>
@@ -105,6 +105,8 @@ void gps_merge_fix(/*@ out @*/struct gps_fix_t *to,
 		   /*@ in @*/struct gps_fix_t *from)
 /* merge new data into an old fix */
 {
+    if ((NULL == to) || (NULL == from))
+	return;
     if ((transfer & TIME_SET)!=0)
 	to->time = from->time;
     if ((transfer & LATLON_SET)!=0) {
@@ -154,6 +156,9 @@ time_t mkgmtime(register struct tm *t)
     result += (year - 1968) / 4;
     result -= (year - 1900) / 100;
     result += (year - 1600) / 400;
+    if ((year % 4) == 0 && ((year % 100) != 0 || (year % 400) == 0) &&
+	(t->tm_mon % MONTHSPERYEAR) < 2)
+	     result--;
     result += t->tm_mday - 1;
     result *= 24;
     result += t->tm_hour;
@@ -180,25 +185,24 @@ double iso8601_to_unix(/*@in@*/char *isotime)
     return (double)mkgmtime(&tm) + usec;
 }
 
-/*@observer@*/char *unix_to_iso8601(double fixtime, /*@ out @*/char isotime[], int len)
+/*@observer@*/char *unix_to_iso8601(double fixtime, /*@ out @*/char isotime[], size_t len)
 /* Unix UTC time to ISO8601, no timezone adjustment */
+/* example: 2007-12-11T23:38:51.0Z */
 {
     struct tm when;
     double integral, fractional;
     time_t intfixtime;
-    size_t slen;
+    char timestr[30];
+    char fractstr[10];
 
     fractional = modf(fixtime, &integral);
     intfixtime = (time_t)integral;
     (void)gmtime_r(&intfixtime, &when);
 
-    (void)strftime(isotime, 28, "%Y-%m-%dT%H:%M:%S", &when);
-    slen = strlen(isotime);
-    (void)snprintf(isotime + slen, (size_t)len, "%.1f", fractional);
-    /*@ -aliasunique @*/
-    (void)memcpy(isotime+slen, isotime+slen+1, strlen(isotime+slen+1));
-    /*@ -aliasunique @*/
-    (void)strlcat(isotime, "Z", 28);
+    (void)strftime(timestr, sizeof(timestr), "%Y-%m-%dT%H:%M:%S", &when);
+    (void)snprintf(fractstr, sizeof(fractstr), "%.1f", fractional);
+    /* add fractional part, ignore leading 0; "0.2" -> ".2" */ 
+    (void)snprintf(isotime, len, "%s%sZ", timestr, fractstr+1);
     return isotime;
 }
 
@@ -294,7 +298,7 @@ double earth_distance(double lat1, double lon1, double lat2, double lon2)
 /*****************************************************************************
 
 Carl Carter of SiRF supplied this algorithm for computing DOPs from 
-a list of visible satellites...
+a list of visible satellites (some typos corrected)...
 
 For satellite n, let az(n) = azimuth angle from North and el(n) be elevation.
 Let:
@@ -319,10 +323,10 @@ And its transpose A~:
 
 Compute the covariance matrix (A~*A)^-1, which is guaranteed symmetric:
 
-    | s(x)^2    s(x)*s(y)  s(x)*s(z)  s(x)*s(t) | 
-    | s(x)*s(y) s(y)^2     s(y)*s(z)  s(y)*s(t) |
-    | s(z)*s(t) s(y)*s(z)  s(z)^2     s(z)*s(t) |
-    | s(x)*s(t) s(y)*s(t)  s(z)*s(t)  s(z)^2    |
+    | s(x)^2    s(x)*s(y)  s(x)*s(z)  s(x)*s(t) |
+    | s(y)*s(x) s(y)^2     s(y)*s(z)  s(y)*s(t) |
+    | s(z)*s(x) s(z)*s(y)  s(z)^2     s(z)*s(t) |
+    | s(t)*s(x) s(t)*s(y)  s(t)*s(z)  s(t)^2    |
 
 Then:
 
@@ -330,7 +334,7 @@ GDOP = sqrt(s(x)^2 + s(y)^2 + s(z)^2 + s(t)^2)
 TDOP = sqrt(s(t)^2)
 PDOP = sqrt(s(x)^2 + s(y)^2 + s(z)^2)
 HDOP = sqrt(s(x)^2 + s(y)^2)
-VDOP = sqrt(s(y)^2)
+VDOP = sqrt(s(z)^2)
 
 Here's how we implement it...
 
@@ -380,7 +384,7 @@ driver.
 ******************************************************************************/
 
 /*@ -fixedformalarray -mustdefine @*/
-static int invert(double mat[4][4], /*@out@*/double inverse[4][4])
+static bool invert(double mat[4][4], /*@out@*/double inverse[4][4])
 {
   // Find all NECESSARY 2x2 subdeterminants
   double Det2_12_01 = mat[1][0]*mat[2][1] - mat[1][1]*mat[2][0];
@@ -444,8 +448,8 @@ static int invert(double mat[4][4], /*@out@*/double inverse[4][4])
 		- mat[0][3]*Det3_123_012;
 
   // Very small determinants probably reflect floating-point fuzz near zero
-  if (fabs(det) == 0.0)
-      return 0;
+  if (fabs(det) < 0.0001)
+      return false;
 
   inverse[0][0] =  Det3_123_123 / det;
   //inverse[0][1] = -Det3_023_123 / det;
@@ -467,7 +471,7 @@ static int invert(double mat[4][4], /*@out@*/double inverse[4][4])
   //inverse[3][2] = -Det3_013_012 / det;
   inverse[3][3] =  Det3_012_012 / det;
 
-  return 1;
+  return true;
 }  
 /*@ +fixedformalarray +mustdefine @*/
 
@@ -476,6 +480,8 @@ gps_mask_t dop(struct gps_data_t *gpsdata)
     double prod[4][4];
     double inv[4][4];
     double satpos[MAXCHANNELS][4];
+    double hdop, vdop, pdop, tdop, gdop;
+    gps_mask_t mask;
     int i, j, k, n;
 
 #ifdef __UNUSED__
@@ -543,13 +549,42 @@ gps_mask_t dop(struct gps_data_t *gpsdata)
 	return 0;
     }
 
+    hdop = sqrt(inv[0][0] + inv[1][1]);
+    vdop = sqrt(inv[2][2]);
+    pdop = sqrt(inv[0][0] + inv[1][1] + inv[2][2]);
+    tdop = sqrt(inv[3][3]);
+    gdop = sqrt(inv[0][0] + inv[1][1] + inv[2][2] + inv[3][3]);
+    mask = 0;
+
+    gpsd_report(LOG_PROG, "DOPS computed/reported: H=%f/%f, V=%f/%f, P=%f/%f, T=%f/%f, G=%f/%f\n",
+		hdop, gpsdata->hdop,
+		vdop, gpsdata->vdop,
+		pdop, gpsdata->pdop,
+		tdop, gpsdata->tdop,
+		gdop, gpsdata->gdop);
+
     /*@ -usedef @*/
-    //gpsdata->hdop = sqrt(inv[0][0] + inv[1][1]);
-    gpsdata->vdop = sqrt(inv[1][1]);
-    gpsdata->pdop = sqrt(inv[0][0] + inv[1][1] + inv[2][2]);
-    gpsdata->tdop = sqrt(inv[3][3]);
-    gpsdata->gdop = sqrt(inv[0][0] + inv[1][1] + inv[2][2] + inv[3][3]);
+    if (isnan(gpsdata->hdop)!=0) {
+	gpsdata->hdop = hdop;
+	mask |= HDOP_SET;
+    }
+    if (isnan(gpsdata->vdop)!=0) {
+	gpsdata->vdop = vdop;
+	mask |= VDOP_SET;
+    }
+    if (isnan(gpsdata->pdop)!=0) {
+	gpsdata->pdop = pdop;
+	mask |= PDOP_SET;
+    }
+    if (isnan(gpsdata->tdop)!=0) {
+	gpsdata->tdop = tdop;
+	mask |= TDOP_SET;
+    }
+    if (isnan(gpsdata->gdop)!=0) {
+	gpsdata->gdop = gdop;
+	mask |= GDOP_SET;
+    }
     /*@ +usedef @*/
 
-    return VDOP_SET | PDOP_SET | TDOP_SET | GDOP_SET;
+    return mask;
 }
