@@ -1,4 +1,4 @@
-/* $Id: italk.c 4651 2008-01-01 14:53:16Z esr $ */
+/* $Id: italk.c 4661 2008-01-19 22:54:23Z garyemiller $ */
 /*
  * Driver for the iTalk binary protocol used by FasTrax
  */
@@ -343,7 +343,7 @@ static void italk_ping(struct gps_device_t *session)
 /* this is everything we export */
 struct gps_type_t italk_binary =
 {
-    .typename       = "iTalk binary",	/* full name of type */
+    .type_name      = "iTalk binary",	/* full name of type */
     .trigger        = NULL,		/* recognize the type */
     .channels       = 12,		/* consumer-grade GPS */
     .probe_wakeup   = NULL,		/* no wakeup to be done before hunt */
@@ -481,7 +481,7 @@ static void itrax_wrap(struct gps_device_t *session)
 
 /*@ -redef @*/
 static struct gps_type_t itrax = {
-    .typename      = "iTrax",		/* full name of type */
+    .type_name     = "iTrax",		/* full name of type */
     .trigger       = "$PFST,OK",	/* tells us to switch to Itrax */
     .channels      = 12,		/* consumer-grade GPS */
     .probe_wakeup  = NULL,		/* no wakeup to be done before hunt */
