@@ -1,4 +1,4 @@
-/* $Id: libgps.c 4070 2006-12-04 12:44:33Z esr $ */
+/* $Id: libgps.c 4377 2007-06-02 14:52:38Z esr $ */
 /* libgps.c -- client interface library for the gpsd daemon */
 #include <sys/time.h>
 #include <stdio.h>
@@ -15,6 +15,10 @@
 #include "gpsd_config.h"
 #include "gpsd.h"
 
+#ifdef HAVE_SETLOCALE
+#include <locale.h>
+#endif
+
 #ifdef S_SPLINT_S
 extern char *strtok_r(char *, const char *, char **);
 #endif /* S_SPLINT_S */
@@ -99,7 +103,10 @@ enum unit gpsd_units(void)
 {
 	char *envu = NULL;
 
- 	if ((envu = getenv("GPSD_UNITS")) != NULL && *envu != '\0') {
+#ifdef HAVE_SETLOCALE
+	(void)setlocale(LC_NUMERIC, "C");
+#endif
+  	if ((envu = getenv("GPSD_UNITS")) != NULL && *envu != '\0') {
 		if (0 == strcasecmp(envu, "imperial")) {
 			return imperial;
 		}
@@ -134,7 +141,7 @@ struct gps_data_t *gps_open(const char *host, const char *port)
     if (!gpsdata)
 	return NULL;
     if (!host)
-	host = "localhost";
+	host = "127.0.0.1";
     if (!port)
 	port = DEFAULT_GPSD_PORT;
 
@@ -281,7 +288,7 @@ static void gps_unpack(char *buf, struct gps_data_t *gpsdata)
 			    (size_t)gpsdata->ndevices,
 			    sizeof(char **));
 			/*@ -nullstate -mustfreefresh @*/
-			gpsdata->devicelist[i=0] = strdup(strtok_r(sp+2, " \r\n", &ns));
+			gpsdata->devicelist[i=0] = strdup(strtok_r(sp+1, " \r\n", &ns));
 			while ((sp = strtok_r(NULL, " \r\n",  &ns)))
 			    gpsdata->devicelist[++i] = strdup(sp);
 			/*@ +nullstate +mustfreefresh @*/
@@ -319,7 +326,7 @@ static void gps_unpack(char *buf, struct gps_data_t *gpsdata)
 				tag, timestr, ept, lat, lon,
 			        alt, eph, epv, track, speed, climb,
 			        epd, eps, epc, mode);
-			if (st == 15) {
+			if (st >= 14) {
 #define DEFAULT(val) (val[0] == '?') ? NAN : atof(val)
 			    /*@ +floatdouble @*/
 			    nf.time = DEFAULT(timestr);
@@ -549,6 +556,7 @@ int gps_query(struct gps_data_t *gpsdata, const char *fmt, ... )
     return gps_poll(gpsdata);
 }
 
+#ifdef HAVE_LIBPTHREAD
 static void *poll_gpsd(void *args) 
 /* helper for the thread launcher */
 {
@@ -600,6 +608,7 @@ int gps_del_callback(struct gps_data_t *gpsdata, pthread_t *handler)
 	(void)gps_query(gpsdata,"w-\n");	/* disable watcher mode */
     return res;
 }
+#endif /* HAVE_LIBPTHREAD */
 
 #ifdef TESTMAIN
 /*
