@@ -1,4 +1,4 @@
-/* $Id: libgpsd_core.c 4600 2007-12-21 02:17:01Z esr $ */
+/* $Id: libgpsd_core.c 4664 2008-01-21 00:37:35Z garyemiller $ */
 /* libgpsd_core.c -- direct access to GPSes on serial or USB devices. */
 #include <sys/time.h>
 #include <sys/ioctl.h>
@@ -23,15 +23,15 @@
 #endif
 
 
-int gpsd_switch_driver(struct gps_device_t *session, char* typename)
+int gpsd_switch_driver(struct gps_device_t *session, char* type_name)
 {
     struct gps_type_t **dp;
 
-    gpsd_report(LOG_PROG, "switch_driver(%s) called...\n", typename);
+    gpsd_report(LOG_PROG, "switch_driver(%s) called...\n", type_name);
     if (session->device_type != NULL && 
-	strcmp(session->device_type->typename, typename) == 0) {
+	strcmp(session->device_type->type_name, type_name) == 0) {
 #ifdef ALLOW_RECONFIGURE
-	gpsd_report(LOG_PROG, "Reconfiguring for %s...\n", session->device_type->typename);
+	gpsd_report(LOG_PROG, "Reconfiguring for %s...\n", session->device_type->type_name);
 	if (session->enable_reconfigure 
 	    	&& session->device_type->configurator != NULL)
 	    session->device_type->configurator(session, 0);
@@ -41,8 +41,8 @@ int gpsd_switch_driver(struct gps_device_t *session, char* typename)
 
     /*@ -compmempass @*/
     for (dp = gpsd_drivers; *dp; dp++)
-	if (strcmp((*dp)->typename, typename) == 0) {
-	    gpsd_report(LOG_PROG, "selecting %s driver...\n", (*dp)->typename);
+	if (strcmp((*dp)->type_name, type_name) == 0) {
+	    gpsd_report(LOG_PROG, "selecting %s driver...\n", (*dp)->type_name);
 	    gpsd_assert_sync(session);
 	    /*@i@*/session->device_type = *dp;
 	    if (!session->context->readonly && session->device_type->probe_subtype != NULL)
@@ -50,13 +50,13 @@ int gpsd_switch_driver(struct gps_device_t *session, char* typename)
 #ifdef ALLOW_RECONFIGURE
 	    if (session->enable_reconfigure 
 			&& session->device_type->configurator != NULL) {
-		gpsd_report(LOG_PROG, "configuring for %s...\n", session->device_type->typename);
+		gpsd_report(LOG_PROG, "configuring for %s...\n", session->device_type->type_name);
 		session->device_type->configurator(session, 0);
 	    }
 #endif /* ALLOW_RECONFIGURE */
 	    return 1;
 	}
-    gpsd_report(LOG_ERROR, "invalid GPS type \"%s\".\n", typename);
+    gpsd_report(LOG_ERROR, "invalid GPS type \"%s\".\n", type_name);
     return 0;
     /*@ +compmempass @*/
 }
@@ -138,6 +138,7 @@ static void *gpsd_ppsmonitor(void *arg)
     pps_device = TIOCM_CTS;
 #endif
 
+    gpsd_report(LOG_PROG, "Create Thread gpsd_ppsmonitor\n");
 
     /* wait for status change on the device's carrier-detect line */
     while (ioctl(session->gpsdata.gps_fd, TIOCMIWAIT, pps_device) == 0) {
@@ -177,38 +178,52 @@ static void *gpsd_ppsmonitor(void *arg)
 	     *
 	     * Some GPS instead output a square wave that is 0.5 Hz and each
 	     * edge denotes the start of a second.
+	     *
+             * A few stupid GPS, like the Furuno GPSClock, output a 1.0 Hz 
+             * square wave where the leading edge is the start of a second
+	     *
+	     * 5Hz GPS (Garmin 18-5Hz) pulses at 5Hz. Set the pulse length to 
+	     * 40ms which gives a 160ms pulse before going high.
+             *
 	     */
 #define timediff(x, y)	(int)((x.tv_sec-y.tv_sec)*1000000+x.tv_usec-y.tv_usec)
 	    cycle = timediff(tv, pulse[state]);
 	    duration = timediff(tv, pulse[state == 0]);
 #undef timediff
 
-	    if (800000 > duration) {
-		/* less than 800mS, duration too short for anything */
-		gpsd_report(LOG_RAW, 
-		    "PPS pulse rejected too short. cycle: %d, duration: %d\n",
-		    cycle, duration);
-#ifdef GPSCLOCK_ENABLE
-		    /*
-		     * Ugly hack to cope with things like the Furuno GPSClock,
-		     * which has the odd property that you have to
-		     * ignore the trailing edge of the PPS.
-		     */
-		    if (state==1 && session->driver.nmea.ignore_trailing_edge) 
-			(void)ntpshm_pps(session, &tv);
-#endif /* GPSCLOCK_ENABLE */
+	    if (cycle > 199000 && cycle < 201000 ) {
+		/* 5Hz cycle */
+		/* looks like 5hz PPS pulse */
+	        if (duration > 45000)
+		    (void)ntpshm_pps(session, &tv);
+		gpsd_report(LOG_RAW, "5Hz PPS pulse. cycle: %d, duration: %d\n",
+			cycle, duration);
 	    } else if (cycle > 999000 && cycle < 1001000 ) {
-		/* looks like PPS pulse */
-		(void)ntpshm_pps(session, &tv);
+		/* looks like PPS pulse or square wave */
+                if (duration > 499000 && duration < 501000 
+		  && session->driver.nmea.ignore_trailing_edge) {
+                    /* looks like 1.0 Hz square wave, ignore trailing edge */
+                    if (state == 1) {
+                         (void)ntpshm_pps(session, &tv);
+                    }
+                } else {
+                    /* looks like PPS pulse */
+                    (void)ntpshm_pps(session, &tv);
+		}
+		gpsd_report(LOG_RAW, "PPS pulse. cycle: %d, duration: %d\n",
+			cycle, duration);
+
 	    } else if (cycle > 1999000 && cycle < 2001000) {
 		/* looks like 0.5 Hz square wave */
 		(void)ntpshm_pps(session, &tv);
+                gpsd_report(LOG_RAW, "PPS square wave. cycle: %d, duration: %d\n",
+			cycle, duration);
 	    } else {
-		    gpsd_report(LOG_RAW, "PPS pulse rejected.  cycle: %d, duration: %d\n",
+                gpsd_report(LOG_INF, "PPS pulse rejected.  cycle: %d, duration: %d\n",
 			cycle, duration);
 	    }
 	} else {
-                gpsd_report(LOG_RAW, "PPS pulse rejected. No fix.\n");
+                gpsd_report(LOG_INF, "PPS pulse rejected. No fix.\n");
 	}
 	/*@ -boolint @*/
 
@@ -233,7 +248,7 @@ int gpsd_activate(struct gps_device_t *session, bool reconfigurable)
 	for (dp = gpsd_drivers; *dp; dp++) {
 	    (void)tcflush(session->gpsdata.gps_fd, TCIOFLUSH);  /* toss stale data */
 	    if ((*dp)->probe_detect!=NULL && (*dp)->probe_detect(session)!=0) {
-		gpsd_report(LOG_PROG, "probe found %s driver...\n", (*dp)->typename);
+		gpsd_report(LOG_PROG, "probe found %s driver...\n", (*dp)->type_name);
 		session->device_type = *dp;
 		gpsd_assert_sync(session);
 		goto foundit;
@@ -308,9 +323,9 @@ char /*@observer@*/ *gpsd_id(/*@in@*/struct gps_device_t *session)
 {
     static char buf[128];
     if ((session == NULL) || (session->device_type == NULL) || 
-	(session->device_type->typename == NULL))
+	(session->device_type->type_name == NULL))
 	return "unknown,";
-    (void)strlcpy(buf, session->device_type->typename, sizeof(buf));
+    (void)strlcpy(buf, session->device_type->type_name, sizeof(buf));
     if (session->subtype[0] != '\0') {
 	(void)strlcat(buf, " ", sizeof(buf));
 	(void)strlcat(buf, session->subtype, sizeof(buf));
