@@ -1,4 +1,4 @@
-/* $Id: packet.c 4644 2007-12-28 03:13:44Z esr $ */
+/* $Id: packet.c 4660 2008-01-17 19:43:57Z esr $ */
 /****************************************************************************
 
 NAME:
@@ -50,7 +50,7 @@ others apart and distinguish them from baud barf.
  * Error handling is brutally simple; any time we see an unexpected
  * character, go to GROUND_STATE and reset the machine (except that a
  * $ in an NMEA payload only resets back to NMEA_DOLLAR state).  Because
- * another good packet will be usually along in less than a second
+ * another good packet will usually be along in less than a second
  * repeating the same data, Boyer-Moore-like attempts to do parallel
  * recognition beyond the headers would make no sense in this
  * application, they'd just add complexity.
@@ -76,7 +76,7 @@ static void nextstate(struct gps_packet_t *lexer,
 #ifdef RTCM104_ENABLE
     enum isgpsstat_t	isgpsstat;
 #endif /* RTCM104_ENABLE */
-/*@ +charint */
+/*@ +charint -casebreak @*/
     switch(lexer->state)
     {
     case GROUND_STATE:
@@ -170,7 +170,6 @@ static void nextstate(struct gps_packet_t *lexer,
 	}
 #endif /* RTCM104_ENABLE */
 	break;
-	/*@ +casebreak @*/
     case COMMENT_BODY:
 	if (c == '\n')
 	    lexer->state = COMMENT_RECOGNIZED;
@@ -440,17 +439,23 @@ static void nextstate(struct gps_packet_t *lexer,
 #if defined(TSIP_ENABLE) || defined(EVERMORE_ENABLE) || defined(GARMIN_ENABLE)
     case DLE_LEADER:
 #ifdef EVERMORE_ENABLE
-	if (c == STX)
+	if (c == STX) {
 	    lexer->state = EVERMORE_LEADER_2;
+	    break;
+	}
 #endif /* EVERMORE_ENABLE */
 #if defined(TSIP_ENABLE) || defined(GARMIN_ENABLE) || defined(NAVCOM_ENABLE)
 	/* garmin is special case of TSIP */
 	/* check last because there's no checksum */
-	if (c >= 0x13)
+	if (c >= 0x13) {
 	    lexer->state = TSIP_PAYLOAD;
-	if (c == DLE)
+	    break;
+	}
+	if (c == DLE) {
 	    lexer->state = GROUND_STATE;
-	break;
+	    break;
+	}
+	// FALL-THRU!!!!! no break here
 #endif /* TSIP_ENABLE */
 #ifdef NAVCOM_ENABLE
     case NAVCOM_LEADER_1:
@@ -755,7 +760,7 @@ static void nextstate(struct gps_packet_t *lexer,
 	break;
 #endif /* RTCM104_ENABLE */
     }
-/*@ -charint */
+/*@ -charint +casebreak @*/
 }
 
 #define STATE_DEBUG
