@@ -1,3 +1,5 @@
+/* $Id: rtcmdecode.c 4069 2006-12-04 12:31:50Z esr $ */
+#include <sys/types.h>
 #include <unistd.h>
 #include <stdlib.h>
 #include <string.h>
@@ -7,6 +9,7 @@
 #include <stdbool.h>
 #include <ctype.h>
 
+#include "gpsd_config.h"
 #include "gpsd.h"
 
 static int verbose = ISGPS_ERRLEVEL_BASE;
@@ -18,7 +21,7 @@ void gpsd_report(int errlevel, const char *fmt, ... )
 	char buf[BUFSIZ];
 	va_list ap;
 
-	strcpy(buf, "rtcmdecode: ");
+	(void)strlcpy(buf, "rtcmdecode: ", BUFSIZ);
 	va_start(ap, fmt) ;
 	(void)vsnprintf(buf + strlen(buf), sizeof(buf)-strlen(buf), fmt, ap);
 	va_end(ap);
@@ -31,21 +34,23 @@ static void decode(FILE *fpin, FILE *fpout)
 /* RTCM-104 bits on fpin to dump format on fpout */
 {
     int             c;
-    struct gps_device_t device;
+    struct gps_packet_t lexer;
+    struct rtcm_t rtcm;
     enum isgpsstat_t res;
     off_t count;
     char buf[BUFSIZ];
 
-    isgps_init(&device);
+    isgps_init(&lexer);
 
     count = 0;
     while ((c = fgetc(fpin)) != EOF) {
-	res = rtcm_decode(&device, (unsigned int)c);
+	res = rtcm_decode(&lexer, (unsigned int)c);
 	if (verbose >= ISGPS_ERRLEVEL_BASE + 3) 
 	    fprintf(fpout, "%08lu: '%c' [%02x] -> %d\n", 
 		   (unsigned long)count++, (isprint(c)?c:'.'), (unsigned)(c & 0xff), res);
 	if (res == ISGPS_MESSAGE) {
-	    rtcm_dump(&device, buf, sizeof(buf));
+	    rtcm_unpack(&rtcm, (char *)lexer.isgps.buf);
+	    rtcm_dump(&rtcm, buf, sizeof(buf));
 	    (void)fputs(buf, fpout);
 	}
     }
@@ -57,9 +62,11 @@ static void pass(FILE *fpin, FILE *fpout)
 /* dump format on stdin to dump format on stdout (self-inversion test) */
 {
     char buf[BUFSIZ];
-    struct gps_device_t session;
+    struct gps_packet_t lexer;
+    struct rtcm_t rtcm;
 
-    memset(&session, 0, sizeof(session));
+    memset(&lexer, 0, sizeof(lexer));
+    memset(&rtcm, 0, sizeof(rtcm));
     while (fgets(buf, (int)sizeof(buf), fpin) != NULL) {
 	int status;
 
@@ -67,16 +74,17 @@ static void pass(FILE *fpin, FILE *fpout)
 	if (buf[0] == '#') {
 	    (void)fputs(buf, fpout);
 	    continue;
-	}
-
-	status = rtcm_undump(&session.gpsdata.rtcm, buf);
+	} 
+	status = rtcm_undump(&rtcm, buf);
 
 	if (status == 0) {
-	    (void)rtcm_repack(&session);
-	    (void)rtcm_unpack(&session);
-	    (void)rtcm_dump(&session, buf, sizeof(buf));
+	    (void)memset(lexer.isgps.buf, 0, sizeof(lexer.isgps.buf));
+	    (void)rtcm_repack(&rtcm, lexer.isgps.buf);
+	    (void)rtcm_unpack(&rtcm, (char *)lexer.isgps.buf);
+	    (void)rtcm_dump(&rtcm, buf, sizeof(buf));
 	    (void)fputs(buf, fpout);
-	    memset(&session, 0, sizeof(session));
+	    memset(&lexer, 0, sizeof(lexer));
+	    memset(&rtcm, 0, sizeof(rtcm));
 	} else if (status < 0) {
 	    (void) fprintf(stderr, "rtcmdecode: bailing out with status %d\n", status);
 	    exit(1);
@@ -90,20 +98,22 @@ static void encode(FILE *fpin, FILE *fpout)
 /* dump format on fpin to RTCM-104 on fpout */
 {
     char buf[BUFSIZ];
-    struct gps_device_t session;
+    struct gps_packet_t lexer;
+    struct rtcm_t rtcm;
 
-    memset(&session, 0, sizeof(session));
+    memset(&lexer, 0, sizeof(lexer));
     while (fgets(buf, (int)sizeof(buf), fpin) != NULL) {
 	int status;
 
-	status = rtcm_undump(&session.gpsdata.rtcm, buf);
+	status = rtcm_undump(&rtcm, buf);
 
 	if (status == 0) {
-	    (void)rtcm_repack(&session);
-	    (void)fwrite(session.driver.isgps.buf, 
+	    (void)memset(lexer.isgps.buf, 0, sizeof(lexer.isgps.buf));
+	    (void)rtcm_repack(&rtcm, lexer.isgps.buf);
+	    (void)fwrite(lexer.isgps.buf, 
 			 sizeof(isgps30bits_t), 
-			 (size_t)session.gpsdata.rtcm.length, fpout);
-	    memset(&session, 0, sizeof(session));
+			 (size_t)rtcm.length, fpout);
+	    memset(&lexer, 0, sizeof(lexer));
 	} else if (status < 0) {
 	    (void) fprintf(stderr, "rtcmdecode: bailing out with status %d\n", status);
 	    exit(1);
@@ -121,28 +131,30 @@ int main(int argc, char **argv)
 
     while ((c = getopt(argc, argv, "dehpVv:")) != EOF) {
 	switch (c) {
-	case 'd':	/* not documented, used for debugging */
+	case 'd':
 	    mode = dodecode;
 	    break;
 
-	case 'e':	/* not documented, used for debugging */
+	case 'e':
 	    mode = doencode;
 	    break;
 
-	case 'h':	/* not documented, used for debugging */
+	case 'h':
 	    striphdr = true;
 	    break;
 
-	case 'p':	/* not documented, used for debugging */
+	case 'p':	/* undocumented, used for regression-testing */
 	    mode = passthrough;
 	    break;
 
-	case 'v':		/* verbose */
+	case 'v':
 	    verbose = ISGPS_ERRLEVEL_BASE + atoi(optarg);
 	    break;
+
 	case 'V':
-	    (void)fprintf(stderr, "SVN ID: $Id: cgps.c 3255 2006-03-02 13:01:15Z esr $ \n");
+	    (void)fprintf(stderr, "SVN ID: $Id: rtcmdecode.c 4069 2006-12-04 12:31:50Z esr $ \n");
 	    exit(0);
+
 	case '?':
 	default:
 	    (void)fputs("rtcmdecode [-v]\n", stderr);
