@@ -1,4 +1,4 @@
-/* $Id: serial.c 4550 2007-12-13 04:14:09Z esr $ */
+/* $Id: serial.c 4666 2008-01-21 01:19:38Z garyemiller $ */
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <unistd.h>
@@ -30,7 +30,9 @@ void gpsd_tty_init(struct gps_device_t *session)
 #ifdef NTPSHM_ENABLE
     /* mark NTPD shared memory segments as unused */
     session->shmindex = -1;
+# ifdef PPS_ENABLE
     session->shmTimeP = -1;
+# endif /* PPS_ENABLE */
 #endif /* NTPSHM_ENABLE */
 }
 
@@ -332,7 +334,13 @@ void gpsd_assert_sync(struct gps_device_t *session)
      * to communicate the navigation message derived and (possibly)
      * 1pps derived time data to ntpd.
      */
-    ntpd_link_activate(session);
+
+    gpsd_report(LOG_INF, "ntpd_link_activate: %d\n",
+      (int)session->shmindex >=0 );
+    /* do not start more than one ntp thread */
+    if (!(session->shmindex >=0))
+       ntpd_link_activate(session);
+
 #endif /* NTPSHM_ENABLE */
 }
 
