@@ -1,4 +1,4 @@
-/* $Id: sirfmon.c 3672 2006-10-27 02:13:44Z esr $ */
+/* $Id: sirfmon.c 4365 2007-05-31 09:52:27Z esr $ */
 /*
  * SiRF packet monitor, originally by Rob Janssen, PE1CHL.
  * Heavily hacked by Eric S. Raymond for use with the gpsd project.
@@ -25,7 +25,6 @@
  */
 #include <sys/types.h>
 #include <stdio.h>
-#include <curses.h>
 #include <stdlib.h>
 #include <string.h>
 #include <math.h>
@@ -43,6 +42,11 @@
 #include <stdbool.h>
 
 #include "gpsd_config.h"
+#ifdef HAVE_NCURSES_H
+#include <ncurses.h>
+#else
+#include <curses.h>
+#endif /* HAVE_NCURSES_H */
 #include "gps.h"	/* for DEFAULT_GPSD_PORT; brings in PI as well */
 
 #define PUT_ORIGIN	-4
@@ -776,7 +780,7 @@ static int readword(void)
 }
 
 /*@ -globstate @*/
-static int readpkt(unsigned char *buf)
+static int readpkt(unsigned char *buf, size_t buflen)
 {
     int byte,len,csum,cnt;
     unsigned char *cp = buf;
@@ -791,6 +795,7 @@ static int readpkt(unsigned char *buf)
 	return EOF;
 
     csum = 0;
+    assert(len < (int)buflen);
     cnt = len;
 
     while (cnt-- > 0) {
@@ -981,7 +986,7 @@ int main (int argc, char **argv)
     /*@ -boolops */
     if (!arg || (arg && !slash) || (arg && colon1 && slash)) {	
 	if (!server)
-	    server = "localhost";
+	    server = "127.0.0.1";
 	devicefd = netlib_connectsock(server, port, "tcp");
 	if (devicefd < 0) {
 	    (void)fprintf(stderr, 
@@ -1047,7 +1052,7 @@ int main (int argc, char **argv)
     (void)wmove(mid2win, 2,1);
     (void)wprintw(mid2win, "Vel:                            m/s                                  climb m/s");
     (void)wmove(mid2win, 3,1);
-    (void)wprintw(mid2win, "Time:                  UTC:                Heading:                  speed m/s");
+    (void)wprintw(mid2win, "Time:                  GPS:                Heading:                  speed m/s");
     (void)wmove(mid2win, 4,1);
     (void)wprintw(mid2win, "Skew:                   TZ:                HDOP:      M1:        M2:    ");
     (void)wmove(mid2win, 5,1);
@@ -1339,7 +1344,7 @@ int main (int argc, char **argv)
 	    (void)sendpkt(buf, 2, device);
 	}
 
-	if ((len = readpkt(buf)) != EOF) {
+	if ((len = readpkt(buf, sizeof(buf))) != EOF) {
 	    decode_sirf(buf,len);
 	}
     }
