@@ -1,3 +1,4 @@
+/* $Id: srecord.c 3666 2006-10-26 23:11:51Z ckuethe $ */
 /*
  * Copyright (c) 2005 Chris Kuethe <chris.kuethe@gmail.com>
  *
@@ -13,9 +14,11 @@
  * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
  * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  */
+#include <sys/types.h>
 #include <stdio.h>
 #include <string.h>
 
+#include "gpsd_config.h"
 #include "gpsd.h"
 
 /*
