@@ -1,4 +1,4 @@
-/* $Id: tsip.c 4651 2008-01-01 14:53:16Z esr $ */
+/* $Id: tsip.c 4661 2008-01-19 22:54:23Z garyemiller $ */
 /*
  * Handle the Trimble TSIP packet format
  * by Rob Janssen, PE1CHL.
@@ -729,7 +729,7 @@ static gps_mask_t tsip_parse_input(struct gps_device_t *session)
 /* this is everything we export */
 struct gps_type_t tsip_binary =
 {
-    .typename       = "Trimble TSIP",	/* full name of type */
+    .type_name      = "Trimble TSIP",	/* full name of type */
     .trigger        = NULL,		/* no trigger */
     .channels       = TSIP_CHANNELS,	/* consumer-grade GPS */
     .probe_wakeup   = NULL,		/* no wakeup to be done before hunt */
